@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2014-2015 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -36,7 +36,6 @@
 #include "as-cleanup.h"
 #include "asb-context.h"
 #include "asb-context-private.h"
-#include "asb-panel.h"
 #include "asb-plugin.h"
 #include "asb-plugin-loader.h"
 #include "asb-task.h"
@@ -48,7 +47,7 @@
 #ifdef HAVE_ALPM
 #include "asb-package-alpm.h"
 #endif
-
+#include "asb-package-cab.h"
 #include "asb-package-deb.h"
 
 typedef struct _AsbContextPrivate	AsbContextPrivate;
@@ -61,25 +60,17 @@ struct _AsbContextPrivate
 	GMutex			 apps_mutex;		/* for ->apps */
 	GPtrArray		*file_globs;		/* of AsbPackage */
 	GPtrArray		*packages;		/* of AsbPackage */
-	AsbPanel		*panel;
 	AsbPluginLoader		*plugin_loader;
-	gboolean		 add_cache_id;
-	gboolean		 hidpi_enabled;
-	gboolean		 embedded_icons;
-	gboolean		 no_net;
+	AsbContextFlags		 flags;
 	guint			 max_threads;
 	guint			 min_icon_size;
 	gdouble			 api_version;
 	gchar			*old_metadata;
-	gchar			*extra_appstream;
-	gchar			*extra_appdata;
-	gchar			*extra_screenshots;
-	gchar			*screenshot_uri;
 	gchar			*log_dir;
-	gchar			*screenshot_dir;
 	gchar			*cache_dir;
 	gchar			*temp_dir;
 	gchar			*output_dir;
+	gchar			*icons_dir;
 	gchar			*basename;
 	gchar			*origin;
 };
@@ -110,22 +101,6 @@ asb_context_realpath (const gchar *path)
 	return real;
 }
 
-/**
- * asb_context_set_no_net:
- * @ctx: A #AsbContext
- * @no_net: if network is disallowed
- *
- * Sets if network access is disallowed.
- *
- * Since: 0.1.0
- **/
-void
-asb_context_set_no_net (AsbContext *ctx, gboolean no_net)
-{
-	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	priv->no_net = no_net;
-}
-
 /**
  * asb_context_set_api_version:
  * @ctx: A #AsbContext
@@ -143,56 +118,24 @@ asb_context_set_api_version (AsbContext *ctx, gdouble api_version)
 }
 
 /**
- * asb_context_set_add_cache_id:
+ * asb_context_set_flags:
  * @ctx: A #AsbContext
- * @add_cache_id: boolean
+ * @flags: #AsbContextFlags, e.g. %ASB_CONTEXT_FLAG_ADD_CACHE_ID
  *
  * Sets if the cache id should be included in the metadata.
  *
- * Since: 0.1.0
+ * Since: 0.3.5
  **/
 void
-asb_context_set_add_cache_id (AsbContext *ctx, gboolean add_cache_id)
+asb_context_set_flags (AsbContext *ctx, AsbContextFlags flags)
 {
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	if (add_cache_id) {
+	if (flags & ASB_CONTEXT_FLAG_ADD_CACHE_ID) {
 		as_store_add_metadata_index (priv->store_failed, "X-CacheID");
 		as_store_add_metadata_index (priv->store_ignore, "X-CacheID");
 		as_store_add_metadata_index (priv->store_old, "X-CacheID");
 	}
-	priv->add_cache_id = add_cache_id;
-}
-
-/**
- * asb_context_set_hidpi_enabled:
- * @ctx: A #AsbContext
- * @hidpi_enabled: boolean
- *
- * Sets if the high DPI icons should be included in the metadata.
- *
- * Since: 0.3.1
- **/
-void
-asb_context_set_hidpi_enabled (AsbContext *ctx, gboolean hidpi_enabled)
-{
-	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	priv->hidpi_enabled = hidpi_enabled;
-}
-
-/**
- * asb_context_set_embedded_icons:
- * @ctx: A #AsbContext
- * @embedded_icons: boolean
- *
- * Sets if the icons should be Base64 embedded in the metadata.
- *
- * Since: 0.3.1
- **/
-void
-asb_context_set_embedded_icons (AsbContext *ctx, gboolean embedded_icons)
-{
-	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	priv->embedded_icons = embedded_icons;
+	priv->flags = flags;
 }
 
 /**
@@ -261,73 +204,6 @@ asb_context_set_old_metadata (AsbContext *ctx, const gchar *old_metadata)
 	priv->old_metadata = asb_context_realpath (old_metadata);
 }
 
-/**
- * asb_context_set_extra_appstream:
- * @ctx: A #AsbContext
- * @extra_appstream: directory name, or %NULL
- *
- * Sets the location of a directory which is used for supplimental AppStream
- * files.
- *
- * Since: 0.1.0
- **/
-void
-asb_context_set_extra_appstream (AsbContext *ctx, const gchar *extra_appstream)
-{
-	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	priv->extra_appstream = asb_context_realpath (extra_appstream);
-}
-
-/**
- * asb_context_set_extra_appdata:
- * @ctx: A #AsbContext
- * @extra_appdata: directory name, or %NULL
- *
- * Sets the location of a directory which is used for supplimental AppData
- * files.
- *
- * Since: 0.1.0
- **/
-void
-asb_context_set_extra_appdata (AsbContext *ctx, const gchar *extra_appdata)
-{
-	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	priv->extra_appdata = asb_context_realpath (extra_appdata);
-}
-
-/**
- * asb_context_set_extra_screenshots:
- * @ctx: A #AsbContext
- * @extra_screenshots: directory name, or %NULL
- *
- * Sets the location of a directory which is used for supplimental screenshot
- * files.
- *
- * Since: 0.1.0
- **/
-void
-asb_context_set_extra_screenshots (AsbContext *ctx, const gchar *extra_screenshots)
-{
-	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	priv->extra_screenshots = asb_context_realpath (extra_screenshots);
-}
-
-/**
- * asb_context_set_screenshot_uri:
- * @ctx: A #AsbContext
- * @screenshot_uri: Remote URI root, e.g. "http://www.mysite.com/screenshots/"
- *
- * Sets the remote screenshot URI for screenshots.
- *
- * Since: 0.1.0
- **/
-void
-asb_context_set_screenshot_uri (AsbContext *ctx, const gchar *screenshot_uri)
-{
-	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	priv->screenshot_uri = g_strdup (screenshot_uri);
-}
-
 /**
  * asb_context_set_log_dir:
  * @ctx: A #AsbContext
@@ -344,22 +220,6 @@ asb_context_set_log_dir (AsbContext *ctx, const gchar *log_dir)
 	priv->log_dir = asb_context_realpath (log_dir);
 }
 
-/**
- * asb_context_set_screenshot_dir:
- * @ctx: A #AsbContext
- * @screenshot_dir: directory
- *
- * Sets the screenshot directory to use when building metadata.
- *
- * Since: 0.2.2
- **/
-void
-asb_context_set_screenshot_dir (AsbContext *ctx, const gchar *screenshot_dir)
-{
-	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	priv->screenshot_dir = g_strdup (screenshot_dir);
-}
-
 /**
  * asb_context_set_cache_dir:
  * @ctx: A #AsbContext
@@ -408,6 +268,22 @@ asb_context_set_output_dir (AsbContext *ctx, const gchar *output_dir)
 	priv->output_dir = g_strdup (output_dir);
 }
 
+/**
+ * asb_context_set_icons_dir:
+ * @ctx: A #AsbContext
+ * @icons_dir: directory
+ *
+ * Sets the icons directory to use when building metadata.
+ *
+ * Since: 0.3.5
+ **/
+void
+asb_context_set_icons_dir (AsbContext *ctx, const gchar *icons_dir)
+{
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
+	priv->icons_dir = g_strdup (icons_dir);
+}
+
 /**
  * asb_context_set_basename:
  * @ctx: A #AsbContext
@@ -441,71 +317,38 @@ asb_context_set_origin (AsbContext *ctx, const gchar *origin)
 }
 
 /**
- * asb_context_get_add_cache_id:
+ * asb_context_get_flags:
  * @ctx: A #AsbContext
  *
- * Gets if the cache_id should be added to the metadata.
+ * Gets the build flags.
  *
- * Returns: boolean
+ * Returns: #AsbContextFlags
  *
- * Since: 0.1.0
+ * Since: 0.3.5
  **/
-gboolean
-asb_context_get_add_cache_id (AsbContext *ctx)
+AsbContextFlags
+asb_context_get_flags (AsbContext *ctx)
 {
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	return priv->add_cache_id;
+	return priv->flags;
 }
 
 /**
- * asb_context_get_hidpi_enabled:
+ * asb_context_get_flag:
  * @ctx: A #AsbContext
+ * @flag: A #AsbContextFlags
  *
- * Gets if the HiDPI support should be used.
+ * Gets one specific build flag.
  *
- * Returns: boolean
+ * Returns: %TRUE if the flag was set
  *
- * Since: 0.3.1
+ * Since: 0.3.5
  **/
 gboolean
-asb_context_get_hidpi_enabled (AsbContext *ctx)
+asb_context_get_flag (AsbContext *ctx, AsbContextFlags flag)
 {
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	return priv->hidpi_enabled;
-}
-
-/**
- * asb_context_get_embedded_icons:
- * @ctx: A #AsbContext
- *
- * Gets if the icons should be embedded in the XML.
- *
- * Returns: boolean
- *
- * Since: 0.3.1
- **/
-gboolean
-asb_context_get_embedded_icons (AsbContext *ctx)
-{
-	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	return priv->embedded_icons;
-}
-
-/**
- * asb_context_get_no_net:
- * @ctx: A #AsbContext
- *
- * Gets if network access is forbidden.
- *
- * Returns: boolean
- *
- * Since: 0.2.5
- **/
-gboolean
-asb_context_get_no_net (AsbContext *ctx)
-{
-	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	return priv->no_net;
+	return (priv->flags & flag) > 0;
 }
 
 /**
@@ -542,6 +385,23 @@ asb_context_get_temp_dir (AsbContext *ctx)
 	return priv->temp_dir;
 }
 
+/**
+ * asb_context_get_cache_dir:
+ * @ctx: A #AsbContext
+ *
+ * Gets the screenshot directory to use
+ *
+ * Returns: directory
+ *
+ * Since: 0.3.6
+ **/
+const gchar *
+asb_context_get_cache_dir (AsbContext *ctx)
+{
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
+	return priv->cache_dir;
+}
+
 /**
  * asb_context_get_plugin_loader:
  * @ctx: A #AsbContext
@@ -576,6 +436,22 @@ asb_context_get_packages (AsbContext *ctx)
 	return priv->packages;
 }
 
+/**
+ * asb_context_add_package:
+ * @ctx: A #AsbContext
+ * @pkg: A #AsbPackage
+ *
+ * Adds a package to the list of packages to be processed
+ *
+ * Since: 0.3.5
+ **/
+void
+asb_context_add_package (AsbContext *ctx, AsbPackage *pkg)
+{
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
+	g_ptr_array_add (priv->packages, g_object_ref (pkg));
+}
+
 /**
  * asb_context_add_filename:
  * @ctx: A #AsbContext
@@ -591,7 +467,6 @@ asb_context_get_packages (AsbContext *ctx)
 gboolean
 asb_context_add_filename (AsbContext *ctx, const gchar *filename, GError **error)
 {
-	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 	_cleanup_object_unref_ AsbPackage *pkg = NULL;
 
 	/* can find in existing metadata */
@@ -609,6 +484,8 @@ asb_context_add_filename (AsbContext *ctx, const gchar *filename, GError **error
 	if (g_str_has_suffix (filename, ".pkg.tar.xz"))
 		pkg = asb_package_alpm_new ();
 #endif
+	if (g_str_has_suffix (filename, ".cab"))
+		pkg = asb_package_cab_new ();
 	if (g_str_has_suffix (filename, ".deb"))
 		pkg = asb_package_deb_new ();
 	if (pkg == NULL) {
@@ -619,11 +496,17 @@ asb_context_add_filename (AsbContext *ctx, const gchar *filename, GError **error
 			     filename);
 		return FALSE;
 	}
-	if (!asb_package_open (pkg, filename, error))
-		return FALSE;
 
 	/* add to array */
-	g_ptr_array_add (priv->packages, g_object_ref (pkg));
+	asb_package_set_filename (pkg, filename);
+
+	/* failed to guess the nevra */
+	if (asb_package_get_name (pkg) == NULL) {
+		if (!asb_package_open (pkg, filename, error))
+			return FALSE;
+	}
+
+	asb_context_add_package (ctx, pkg);
 	return TRUE;
 }
 
@@ -644,55 +527,6 @@ asb_context_get_file_globs (AsbContext *ctx)
 	return priv->file_globs;
 }
 
-/**
- * asb_context_load_extra_screenshots:
- **/
-static gboolean
-asb_context_load_extra_screenshots (AsbContext *ctx, AsApp *app, GError **error)
-{
-	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	const gchar *tmp;
-	_cleanup_dir_close_ GDir *dir = NULL;
-	_cleanup_free_ gchar *path = NULL;
-	_cleanup_object_unref_ AsbApp *app_build = NULL;
-	_cleanup_object_unref_ AsbPackage *pkg = NULL;
-
-	/* are there any extra screenshots for this app */
-	if (priv->extra_screenshots == NULL ||
-	    priv->screenshot_uri == NULL ||
-	    priv->screenshot_dir == NULL)
-		return TRUE;
-	path = g_build_filename (priv->extra_screenshots, as_app_get_id_filename (app), NULL);
-	if (!g_file_test (path, G_FILE_TEST_EXISTS))
-		return TRUE;
-
-	/* create a virtual package */
-	pkg = asb_package_new ();
-	asb_package_set_name (pkg, as_app_get_id (app));
-	asb_package_set_config (pkg, "MirrorURI", priv->screenshot_uri);
-	asb_package_set_config (pkg, "ScreenshotDir", priv->screenshot_dir);
-
-	/* create a new AsbApp and add all the extra screenshots */
-	app_build = asb_app_new (pkg, as_app_get_id (app));
-	asb_app_set_hidpi_enabled (app_build, priv->hidpi_enabled);
-	as_app_subsume_full (AS_APP (app_build), app,
-			     AS_APP_SUBSUME_FLAG_NO_OVERWRITE);
-	dir = g_dir_open (path, 0, error);
-	if (dir == NULL)
-		return FALSE;
-	while ((tmp = g_dir_read_name (dir)) != NULL) {
-		_cleanup_free_ gchar *filename = NULL;
-		filename = g_build_filename (path, tmp, NULL);
-		if (!asb_app_add_screenshot_source (app_build, filename, error))
-			return FALSE;
-	}
-	as_app_subsume_full (app, AS_APP (app_build),
-			     AS_APP_SUBSUME_FLAG_NO_OVERWRITE);
-
-	/* save all the screenshots to disk */
-	return asb_app_save_resources (app_build, ASB_APP_SAVE_FLAG_SCREENSHOTS, error);
-}
-
 /**
  * asb_context_setup:
  * @ctx: A #AsbContext
@@ -707,15 +541,10 @@ asb_context_load_extra_screenshots (AsbContext *ctx, AsApp *app, GError **error)
 gboolean
 asb_context_setup (AsbContext *ctx, GError **error)
 {
-	AsApp *app;
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	GList *l;
-	guint i;
-	guint sizes[] = { AS_IMAGE_NORMAL_WIDTH,    AS_IMAGE_NORMAL_HEIGHT,
-			  AS_IMAGE_THUMBNAIL_WIDTH, AS_IMAGE_THUMBNAIL_HEIGHT,
-			  AS_IMAGE_LARGE_WIDTH,     AS_IMAGE_LARGE_HEIGHT,
-			  0 };
 	_cleanup_free_ gchar *icons_dir = NULL;
+	_cleanup_free_ gchar *screenshot_dir1 = NULL;
+	_cleanup_free_ gchar *screenshot_dir2 = NULL;
 
 	/* required stuff set */
 	if (priv->origin == NULL) {
@@ -732,6 +561,13 @@ asb_context_setup (AsbContext *ctx, GError **error)
 				     "output_dir not set!");
 		return FALSE;
 	}
+	if (priv->icons_dir == NULL) {
+		g_set_error_literal (error,
+				     ASB_PLUGIN_ERROR,
+				     ASB_PLUGIN_ERROR_FAILED,
+				     "icons_dir not set!");
+		return FALSE;
+	}
 	if (priv->temp_dir == NULL) {
 		g_set_error_literal (error,
 				     ASB_PLUGIN_ERROR,
@@ -748,11 +584,13 @@ asb_context_setup (AsbContext *ctx, GError **error)
 	}
 
 	/* create temp space */
-	if (!asb_utils_ensure_exists_and_empty (priv->temp_dir, error))
-		return FALSE;
 	if (!asb_utils_ensure_exists (priv->output_dir, error))
 		return FALSE;
-	if (!asb_utils_ensure_exists (priv->cache_dir, error))
+	screenshot_dir1 = g_build_filename (priv->temp_dir, "screenshots", NULL);
+	if (!asb_utils_ensure_exists_and_empty (screenshot_dir1, error))
+		return FALSE;
+	screenshot_dir2 = g_build_filename (priv->cache_dir, "screenshots", NULL);
+	if (!asb_utils_ensure_exists (screenshot_dir2, error))
 		return FALSE;
 	if (priv->log_dir != NULL) {
 		if (!asb_utils_ensure_exists (priv->log_dir, error))
@@ -760,51 +598,19 @@ asb_context_setup (AsbContext *ctx, GError **error)
 	}
 
 	/* icons is nuked; we can re-decompress from the -icons.tar.gz */
-	icons_dir = g_build_filename (priv->temp_dir, "icons", NULL);
-	if (!asb_utils_ensure_exists (icons_dir, error))
+	if (!asb_utils_ensure_exists (priv->icons_dir, error))
 		return FALSE;
-	if (priv->hidpi_enabled) {
+	if (priv->flags & ASB_CONTEXT_FLAG_HIDPI_ICONS) {
 		_cleanup_free_ gchar *icons_dir_hidpi = NULL;
 		_cleanup_free_ gchar *icons_dir_lodpi = NULL;
-		icons_dir_lodpi = g_build_filename (icons_dir, "64x64", NULL);
+		icons_dir_lodpi = g_build_filename (priv->icons_dir, "64x64", NULL);
 		if (!asb_utils_ensure_exists (icons_dir_lodpi, error))
 			return FALSE;
-		icons_dir_hidpi = g_build_filename (icons_dir, "128x128", NULL);
+		icons_dir_hidpi = g_build_filename (priv->icons_dir, "128x128", NULL);
 		if (!asb_utils_ensure_exists (icons_dir_hidpi, error))
 			return FALSE;
 	}
 
-	/* create all the screenshot sizes */
-	if (priv->screenshot_dir != NULL) {
-		_cleanup_free_ gchar *ss_src = NULL;
-		ss_src = g_build_filename (priv->screenshot_dir,
-					   "source", NULL);
-		if (!asb_utils_ensure_exists (ss_src, error))
-			return FALSE;
-		for (i = 0; sizes[i] != 0; i += 2) {
-			_cleanup_free_ gchar *size_str = NULL;
-			_cleanup_free_ gchar *ss_dir = NULL;
-			size_str = g_strdup_printf ("%ix%i",
-						    sizes[i],
-						    sizes[i+1]);
-			ss_dir = g_build_filename (priv->screenshot_dir,
-						   size_str, NULL);
-			if (!asb_utils_ensure_exists (ss_dir, error))
-				return FALSE;
-		}
-		for (i = 0; sizes[i] != 0 && priv->hidpi_enabled; i += 2) {
-			_cleanup_free_ gchar *size_str = NULL;
-			_cleanup_free_ gchar *ss_dir = NULL;
-			size_str = g_strdup_printf ("%ix%i",
-						    sizes[i] * 2,
-						    sizes[i+1] * 2);
-			ss_dir = g_build_filename (priv->screenshot_dir,
-						   size_str, NULL);
-			if (!asb_utils_ensure_exists (ss_dir, error))
-				return FALSE;
-		}
-	}
-
 	/* decompress the icons */
 	if (priv->old_metadata != NULL) {
 		_cleanup_free_ gchar *icons_fn = NULL;
@@ -813,7 +619,7 @@ asb_context_setup (AsbContext *ctx, GError **error)
 					    priv->basename);
 		if (g_file_test (icons_fn, G_FILE_TEST_EXISTS)) {
 			if (!asb_utils_explode (icons_fn,
-						icons_dir,
+						priv->icons_dir,
 						NULL,
 						error))
 				return FALSE;
@@ -827,21 +633,6 @@ asb_context_setup (AsbContext *ctx, GError **error)
 	/* get a cache of the file globs */
 	priv->file_globs = asb_plugin_loader_get_globs (priv->plugin_loader);
 
-	/* add any extra applications and resize screenshots */
-	if (priv->extra_appstream != NULL &&
-	    g_file_test (priv->extra_appstream, G_FILE_TEST_EXISTS)) {
-		if (!asb_utils_add_apps_from_dir (&priv->apps,
-						  priv->extra_appstream,
-						  error))
-			return FALSE;
-		for (l = priv->apps; l != NULL; l = l->next) {
-			app = AS_APP (l->data);
-			if (!asb_context_load_extra_screenshots (ctx, app, error))
-				return FALSE;
-		}
-		g_print ("Added extra %i apps\n", g_list_length (priv->apps));
-	}
-
 	/* add old metadata */
 	if (priv->old_metadata != NULL) {
 		_cleanup_free_ gchar *builder_id = NULL;
@@ -933,15 +724,42 @@ asb_context_write_icons (AsbContext *ctx,
 {
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 	_cleanup_free_ gchar *filename = NULL;
-	_cleanup_free_ gchar *icons_dir = NULL;
 
-	icons_dir = g_build_filename (temp_dir, "icons", NULL);
-	if (!g_file_test (icons_dir, G_FILE_TEST_EXISTS))
+	/* not enabled */
+	if (priv->flags & ASB_CONTEXT_FLAG_UNCOMPRESSED_ICONS)
+		return TRUE;
+
+	if (!g_file_test (priv->icons_dir, G_FILE_TEST_EXISTS))
 		return TRUE;
 	filename = g_strdup_printf ("%s/%s-icons.tar.gz",
 				    priv->output_dir, priv->basename);
 	g_print ("Writing %s...\n", filename);
-	return asb_utils_write_archive_dir (filename, icons_dir, error);
+	return asb_utils_write_archive_dir (filename, priv->icons_dir, error);
+}
+
+/**
+ * asb_context_write_screenshots:
+ **/
+static gboolean
+asb_context_write_screenshots (AsbContext *ctx,
+			       const gchar *temp_dir,
+			       GError **error)
+{
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
+	_cleanup_free_ gchar *filename = NULL;
+	_cleanup_free_ gchar *screenshot_dir = NULL;
+
+	/* not enabled */
+	if (priv->flags & ASB_CONTEXT_FLAG_UNCOMPRESSED_ICONS)
+		return TRUE;
+
+	screenshot_dir = g_build_filename (temp_dir, "screenshots", NULL);
+	if (!g_file_test (screenshot_dir, G_FILE_TEST_EXISTS))
+		return TRUE;
+	filename = g_strdup_printf ("%s/%s-screenshots.tar",
+				    priv->output_dir, priv->basename);
+	g_print ("Writing %s...\n", filename);
+	return asb_utils_write_archive_dir (filename, screenshot_dir, error);
 }
 
 /**
@@ -995,7 +813,7 @@ asb_context_write_xml (AsbContext *ctx, GError **error)
 	g_print ("Writing %s...\n", filename);
 	as_store_set_origin (store, priv->origin);
 	as_store_set_api_version (store, priv->api_version);
-	if (priv->add_cache_id) {
+	if (priv->flags & ASB_CONTEXT_FLAG_ADD_CACHE_ID) {
 		_cleanup_free_ gchar *builder_id = asb_utils_get_builder_id ();
 		as_store_set_builder_id (store, builder_id);
 	}
@@ -1018,7 +836,7 @@ asb_context_convert_icons (AsbContext *ctx, GError **error)
 	GList *l;
 
 	/* not enabled */
-	if (!asb_context_get_embedded_icons (ctx))
+	if ((priv->flags & ASB_CONTEXT_FLAG_EMBEDDED_ICONS) == 0)
 		return TRUE;
 
 	/* convert each one before saving resources */
@@ -1217,6 +1035,10 @@ asb_context_write_xml_fail (AsbContext *ctx, GError **error)
 	_cleanup_free_ gchar *filename = NULL;
 	_cleanup_object_unref_ GFile *file = NULL;
 
+	/* no need to create */
+	if ((priv->flags & ASB_CONTEXT_FLAG_INCLUDE_FAILED) == 0)
+		return TRUE;
+
 	for (l = priv->apps; l != NULL; l = l->next) {
 		app = AS_APP (l->data);
 		if (!ASB_IS_APP (app))
@@ -1238,7 +1060,7 @@ asb_context_write_xml_fail (AsbContext *ctx, GError **error)
 	basename_failed = g_strdup_printf ("%s-failed", priv->origin);
 	as_store_set_origin (priv->store_failed, basename_failed);
 	as_store_set_api_version (priv->store_failed, priv->api_version);
-	if (priv->add_cache_id) {
+	if (priv->flags & ASB_CONTEXT_FLAG_ADD_CACHE_ID) {
 		_cleanup_free_ gchar *builder_id = asb_utils_get_builder_id ();
 		as_store_set_builder_id (priv->store_failed, builder_id);
 	}
@@ -1262,7 +1084,7 @@ asb_context_write_xml_ignore (AsbContext *ctx, GError **error)
 	_cleanup_object_unref_ GFile *file = NULL;
 
 	/* no need to create */
-	if (!priv->add_cache_id)
+	if ((priv->flags & ASB_CONTEXT_FLAG_ADD_CACHE_ID) == 0)
 		return TRUE;
 
 	/* the store is already populated */
@@ -1274,7 +1096,7 @@ asb_context_write_xml_ignore (AsbContext *ctx, GError **error)
 	basename_cache = g_strdup_printf ("%s-ignore", priv->origin);
 	as_store_set_origin (priv->store_ignore, basename_cache);
 	as_store_set_api_version (priv->store_ignore, priv->api_version);
-	if (priv->add_cache_id) {
+	if (priv->flags & ASB_CONTEXT_FLAG_ADD_CACHE_ID) {
 		_cleanup_free_ gchar *builder_id = asb_utils_get_builder_id ();
 		as_store_set_builder_id (priv->store_ignore, builder_id);
 	}
@@ -1308,13 +1130,21 @@ asb_context_disable_older_pkgs (AsbContext *ctx)
 		key = asb_package_get_name (pkg);
 		if (key == NULL)
 			continue;
-		found = g_hash_table_lookup (newest, key);
-		if (found != NULL) {
-			if (asb_package_compare (pkg, found) <= 0) {
-				asb_package_set_enabled (pkg, FALSE);
-				continue;
+		switch (asb_package_get_kind (pkg)) {
+		case ASB_PACKAGE_KIND_DEFAULT:
+		case ASB_PACKAGE_KIND_BUNDLE:
+			found = g_hash_table_lookup (newest, key);
+			if (found != NULL) {
+				if (asb_package_compare (pkg, found) <= 0) {
+					asb_package_set_enabled (pkg, FALSE);
+					continue;
+				}
+				asb_package_set_enabled (found, FALSE);
 			}
-			asb_package_set_enabled (found, FALSE);
+			break;
+		default:
+			/* allow multiple versions */
+			break;
 		}
 		g_hash_table_insert (newest, g_strdup (key), g_object_ref (pkg));
 	}
@@ -1328,6 +1158,7 @@ asb_context_disable_multiarch_pkgs (AsbContext *ctx)
 {
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 	AsbPackage *pkg;
+	const gchar *arch;
 	gboolean found_arch = FALSE;
 	guint i;
 
@@ -1345,16 +1176,21 @@ asb_context_disable_multiarch_pkgs (AsbContext *ctx)
 	/* disable any alternate-arch packages */
 	for (i = 0; i < priv->packages->len; i++) {
 		pkg = ASB_PACKAGE (g_ptr_array_index (priv->packages, i));
-		if (g_strcmp0 (asb_package_get_arch (pkg), "x86_64") != 0 &&
-		    g_strcmp0 (asb_package_get_arch (pkg), "noarch") != 0)
+		arch = asb_package_get_arch (pkg);
+		if (arch == NULL)
+			continue;
+		if (g_strcmp0 (arch, "x86_64") != 0 &&
+		    g_strcmp0 (arch, "noarch") != 0) {
+			g_debug ("disabling alternate-arch %s",
+				 asb_package_get_filename (pkg));
 			asb_package_set_enabled (pkg, FALSE);
+		}
 	}
 }
 
 /**
  * asb_context_process:
  * @ctx: A #AsbContext
- * @flags: Some #AsbContextProcessFlags, e.g. %AS_CONTEXT_PARSE_FLAG_IGNORE_MISSING_PARENTS
  * @error: A #GError or %NULL
  *
  * Processes all the packages that have been added to the context.
@@ -1364,7 +1200,7 @@ asb_context_disable_multiarch_pkgs (AsbContext *ctx)
  * Since: 0.1.0
  **/
 gboolean
-asb_context_process (AsbContext *ctx, AsbContextProcessFlags flags, GError **error)
+asb_context_process (AsbContext *ctx, GError **error)
 {
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 	AsbPackage *pkg;
@@ -1389,7 +1225,6 @@ asb_context_process (AsbContext *ctx, AsbContextProcessFlags flags, GError **err
 
 	/* add each package */
 	g_print ("Processing packages...\n");
-	asb_panel_set_job_total (priv->panel, priv->packages->len);
 	tasks = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	for (i = 0; i < priv->packages->len; i++) {
 		pkg = g_ptr_array_index (priv->packages, i);
@@ -1404,25 +1239,24 @@ asb_context_process (AsbContext *ctx, AsbContextProcessFlags flags, GError **err
 		}
 
 		/* set locations of external resources */
-		asb_package_set_config (pkg, "AppDataExtra", priv->extra_appdata);
-		asb_package_set_config (pkg, "ScreenshotsExtra", priv->extra_screenshots);
-		asb_package_set_config (pkg, "MirrorURI", priv->screenshot_uri);
 		asb_package_set_config (pkg, "LogDir", priv->log_dir);
-		asb_package_set_config (pkg, "ScreenshotDir", priv->screenshot_dir);
-		asb_package_set_config (pkg, "CacheDir", priv->cache_dir);
 		asb_package_set_config (pkg, "TempDir", priv->temp_dir);
+		asb_package_set_config (pkg, "IconsDir", priv->icons_dir);
 		asb_package_set_config (pkg, "OutputDir", priv->output_dir);
 
 		/* create task */
 		task = asb_task_new (ctx);
-		asb_task_set_id (task, i);
 		asb_task_set_package (task, pkg);
-		asb_task_set_panel (task, priv->panel);
 		g_ptr_array_add (tasks, task);
 
-		/* add task to pool */
-		if (!g_thread_pool_push (pool, task, error))
-			return FALSE;
+		/* run the task */
+		if (priv->max_threads == 1) {
+			if (!asb_task_process (task, error))
+				return FALSE;
+		} else {
+			if (!g_thread_pool_push (pool, task, error))
+				return FALSE;
+		}
 	}
 
 	/* wait for them to finish */
@@ -1433,11 +1267,11 @@ asb_context_process (AsbContext *ctx, AsbContextProcessFlags flags, GError **err
 	asb_plugin_loader_merge (priv->plugin_loader, priv->apps);
 
 	/* print any warnings */
-	if ((flags & AS_CONTEXT_PARSE_FLAG_IGNORE_MISSING_INFO) == 0) {
+	if ((priv->flags & ASB_CONTEXT_FLAG_IGNORE_MISSING_INFO) == 0) {
 		if (!asb_context_detect_missing_data (ctx, error))
 			return FALSE;
 	}
-	if ((flags & AS_CONTEXT_PARSE_FLAG_IGNORE_MISSING_PARENTS) == 0) {
+	if ((priv->flags & ASB_CONTEXT_FLAG_IGNORE_MISSING_PARENTS) == 0) {
 		if (!asb_context_detect_missing_parents (ctx, error))
 			return FALSE;
 	}
@@ -1473,6 +1307,13 @@ asb_context_process (AsbContext *ctx, AsbContextProcessFlags flags, GError **err
 	if (!ret)
 		return FALSE;
 
+	/* write screenshots archive */
+	ret = asb_context_write_screenshots (ctx,
+					     priv->temp_dir,
+					     error);
+	if (!ret)
+		return FALSE;
+
 	/* ensure all packages are flushed */
 	for (i = 0; i < priv->packages->len; i++) {
 		pkg = g_ptr_array_index (priv->packages, i);
@@ -1585,7 +1426,7 @@ asb_context_add_app_ignore (AsbContext *ctx, AsbPackage *pkg)
 	_cleanup_ptrarray_unref_ GPtrArray *apps = NULL;
 
 	/* only do this when we are using a cache-id */
-	if (!priv->add_cache_id)
+	if ((priv->flags & ASB_CONTEXT_FLAG_ADD_CACHE_ID) == 0)
 		return;
 
 	/* check not already added a dummy application for this package */
@@ -1630,7 +1471,6 @@ asb_context_finalize (GObject *object)
 	g_object_unref (priv->store_ignore);
 	g_object_unref (priv->store_old);
 	g_object_unref (priv->plugin_loader);
-	g_object_unref (priv->panel);
 	g_ptr_array_unref (priv->packages);
 	g_list_foreach (priv->apps, (GFunc) g_object_unref, NULL);
 	g_list_free (priv->apps);
@@ -1638,15 +1478,11 @@ asb_context_finalize (GObject *object)
 		g_ptr_array_unref (priv->file_globs);
 	g_mutex_clear (&priv->apps_mutex);
 	g_free (priv->old_metadata);
-	g_free (priv->extra_appstream);
-	g_free (priv->extra_appdata);
-	g_free (priv->extra_screenshots);
-	g_free (priv->screenshot_uri);
 	g_free (priv->log_dir);
-	g_free (priv->screenshot_dir);
 	g_free (priv->cache_dir);
 	g_free (priv->temp_dir);
 	g_free (priv->output_dir);
+	g_free (priv->icons_dir);
 	g_free (priv->basename);
 	g_free (priv->origin);
 
@@ -1662,7 +1498,6 @@ asb_context_init (AsbContext *ctx)
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 
 	priv->plugin_loader = asb_plugin_loader_new (ctx);
-	priv->panel = asb_panel_new ();
 	priv->packages = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	g_mutex_init (&priv->apps_mutex);
 	priv->store_failed = as_store_new ();
