@@ -59,7 +59,8 @@ asb_package_rpm_finalize (GObject *object)
 	AsbPackageRpm *pkg = ASB_PACKAGE_RPM (object);
 	AsbPackageRpmPrivate *priv = GET_PRIVATE (pkg);
 
-	headerFree (priv->h);
+	if (priv->h != NULL)
+		headerFree (priv->h);
 
 	G_OBJECT_CLASS (asb_package_rpm_parent_class)->finalize (object);
 }
@@ -419,9 +420,7 @@ asb_package_rpm_ensure_deps (AsbPackage *pkg, GError **error)
 	gboolean ret = TRUE;
 	gchar *tmp;
 	gint rc;
-	guint i = 0;
 	rpmtd td = NULL;
-	_cleanup_strv_free_ gchar **deps = NULL;
 
 	/* read out the dep list */
 	td = rpmtdNew ();
@@ -435,21 +434,19 @@ asb_package_rpm_ensure_deps (AsbPackage *pkg, GError **error)
 			     asb_package_get_filename (pkg));
 		goto out;
 	}
-	deps = g_new0 (gchar *, rpmtdCount (td) + 1);
 	while (rpmtdNext (td) != -1) {
+		_cleanup_free_ gchar *dep_no_qual = NULL;
 		dep = rpmtdGetString (td);
 		if (g_str_has_prefix (dep, "rpmlib"))
 			continue;
 		if (g_strcmp0 (dep, "/bin/sh") == 0)
 			continue;
-		deps[i] = g_strdup (dep);
-		tmp = g_strstr_len (deps[i], -1, "(");
+		dep_no_qual = g_strdup (dep);
+		tmp = g_strstr_len (dep_no_qual, -1, "(");
 		if (tmp != NULL)
 			*tmp = '\0';
-		/* TODO: deduplicate */
-		i++;
+		asb_package_add_dep (pkg, dep_no_qual);
 	}
-	asb_package_set_deps (pkg, deps);
 out:
 	rpmtdFreeData (td);
 	rpmtdFree (td);
@@ -548,6 +545,20 @@ asb_package_rpm_strerror (rpmRC rc)
 	return str;
 }
 
+/**
+ * asb_package_rpm_close:
+ **/
+static gboolean
+asb_package_rpm_close (AsbPackage *pkg, GError **error)
+{
+	AsbPackageRpm *pkg_rpm = ASB_PACKAGE_RPM (pkg);
+	AsbPackageRpmPrivate *priv = GET_PRIVATE (pkg_rpm);
+	if (priv->h != NULL)
+		headerFree (priv->h);
+	priv->h = NULL;
+	return TRUE;
+}
+
 /**
  * asb_package_rpm_open:
  **/
@@ -660,6 +671,7 @@ asb_package_rpm_class_init (AsbPackageRpmClass *klass)
 
 	object_class->finalize = asb_package_rpm_finalize;
 	package_class->open = asb_package_rpm_open;
+	package_class->close = asb_package_rpm_close;
 	package_class->ensure = asb_package_rpm_ensure;
 	package_class->compare = asb_package_rpm_compare;
 }
