@@ -38,9 +38,13 @@
 typedef struct _AsbPackagePrivate	AsbPackagePrivate;
 struct _AsbPackagePrivate
 {
+	AsbPackageKind	 kind;
 	gboolean	 enabled;
+	gboolean	 is_open;
 	gchar		**filelist;
-	gchar		**deps;
+	guint		 filelist_refcount;
+	GPtrArray	*deps;
+	guint		 deps_refcount;
 	gchar		*filename;
 	gchar		*basename;
 	gchar		*name;
@@ -81,7 +85,7 @@ asb_package_finalize (GObject *object)
 
 	g_mutex_clear (&priv->mutex_log);
 	g_strfreev (priv->filelist);
-	g_strfreev (priv->deps);
+	g_ptr_array_unref (priv->deps);
 	g_free (priv->filename);
 	g_free (priv->basename);
 	g_free (priv->name);
@@ -113,8 +117,9 @@ asb_package_init (AsbPackage *pkg)
 {
 	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
 	priv->enabled = TRUE;
-	priv->log = g_string_sized_new (1024);
+	priv->log = g_string_new (NULL);
 	priv->timer = g_timer_new ();
+	priv->deps = g_ptr_array_new_with_free_func (g_free);
 	priv->configs = g_hash_table_new_full (g_str_hash, g_str_equal,
 					       g_free, g_free);
 	priv->releases = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
@@ -280,6 +285,23 @@ asb_package_get_filename (AsbPackage *pkg)
 	return priv->filename;
 }
 
+/**
+ * asb_package_get_kind:
+ * @pkg: A #AsbPackage
+ *
+ * Gets the kind of the package.
+ *
+ * Returns: a #AsbPackageKind
+ *
+ * Since: 0.2.5
+ **/
+AsbPackageKind
+asb_package_get_kind (AsbPackage *pkg)
+{
+	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
+	return priv->kind;
+}
+
 /**
  * asb_package_get_basename:
  * @pkg: A #AsbPackage
@@ -314,6 +336,23 @@ asb_package_get_name (AsbPackage *pkg)
 	return priv->name;
 }
 
+/**
+ * asb_package_get_version:
+ * @pkg: A #AsbPackage
+ *
+ * Gets the package version
+ *
+ * Returns: utf8 string
+ *
+ * Since: 0.3.5
+ **/
+const gchar *
+asb_package_get_version (AsbPackage *pkg)
+{
+	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
+	return priv->version;
+}
+
 /**
  * asb_package_get_arch:
  * @pkg: A #AsbPackage
@@ -441,15 +480,31 @@ asb_package_get_filelist (AsbPackage *pkg)
  *
  * Returns: (transfer none) (element-type utf8): deplist
  *
- * Since: 0.1.0
+ * Since: 0.3.5
  **/
-gchar **
+GPtrArray *
 asb_package_get_deps (AsbPackage *pkg)
 {
 	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
 	return priv->deps;
 }
 
+/**
+ * asb_package_set_kind:
+ * @pkg: A #AsbPackage
+ * @kind: A #AsbPackageKind
+ *
+ * Sets the package kind.
+ *
+ * Since: 0.2.5
+ **/
+void
+asb_package_set_kind (AsbPackage *pkg, AsbPackageKind kind)
+{
+	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
+	priv->kind = kind;
+}
+
 /**
  * asb_package_set_name:
  * @pkg: A #AsbPackage
@@ -622,20 +677,19 @@ asb_package_set_source_pkgname (AsbPackage *pkg, const gchar *source_pkgname)
 }
 
 /**
- * asb_package_set_deps:
+ * asb_package_add_dep:
  * @pkg: A #AsbPackage
- * @deps: package deps
+ * @dep: package dep
  *
- * Sets the package dependancies.
+ * Add a package dependancy.
  *
- * Since: 0.1.0
+ * Since: 0.3.5
  **/
 void
-asb_package_set_deps (AsbPackage *pkg, gchar **deps)
+asb_package_add_dep (AsbPackage *pkg, const gchar *dep)
 {
 	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
-	g_strfreev (priv->deps);
-	priv->deps = g_strdupv (deps);
+	g_ptr_array_add (priv->deps, g_strdup (dep));
 }
 
 /**
@@ -758,6 +812,73 @@ asb_package_class_init (AsbPackageClass *klass)
 	object_class->finalize = asb_package_finalize;
 }
 
+/**
+ * asb_package_guess_from_filename:
+ **/
+static void
+asb_package_guess_from_filename (AsbPackage *pkg)
+{
+	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
+	_cleanup_free_ gchar *tmp = NULL;
+	gchar *at;
+
+	/* remove .rpm extension */
+	tmp = g_strdup (priv->filename);
+	at = g_strrstr (tmp, ".rpm");
+	if (at == NULL)
+		return;
+	*at = '\0';
+
+	/* get arch */
+	at = g_strrstr (tmp, ".");
+	if (at == NULL)
+		return;
+	priv->arch = g_strdup (at + 1);
+	*at = '\0';
+
+	/* get release */
+	at = g_strrstr (tmp, "-");
+	if (at == NULL)
+		return;
+	priv->release = g_strdup (at + 1);
+	*at = '\0';
+
+	/* get version */
+	at = g_strrstr (tmp, "-");
+	if (at == NULL)
+		return;
+	priv->version = g_strdup (at + 1);
+	*at = '\0';
+
+	/* get name */
+	at = g_strrstr (tmp, "/");
+	if (at == NULL)
+		return;
+	priv->name = g_strdup (at + 1);
+}
+
+/**
+ * asb_package_set_filename:
+ * @pkg: A #AsbPackage
+ * @filename: package filename
+ *
+ * Sets the package filename.
+ *
+ * Since: 0.3.5
+ **/
+void
+asb_package_set_filename (AsbPackage *pkg, const gchar *filename)
+{
+	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
+	g_free (priv->basename);
+	g_free (priv->filename);
+	priv->basename = g_path_get_basename (filename);
+	priv->filename = g_strdup (filename);
+
+	/* this only works for correctly formatted file names */
+	asb_package_guess_from_filename (pkg);
+}
+
 /**
  * asb_package_open:
  * @pkg: A #AsbPackage
@@ -778,9 +899,14 @@ asb_package_open (AsbPackage *pkg, const gchar *filename, GError **error)
 	AsbPackageClass *klass = ASB_PACKAGE_GET_CLASS (pkg);
 	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
 
-	/* cache here */
-	priv->filename = g_strdup (filename);
-	priv->basename = g_path_get_basename (filename);
+	/* already open */
+	if (priv->is_open)
+		return TRUE;
+	priv->is_open = TRUE;
+
+	/* save filename if not already set */
+	if (priv->filename == NULL)
+		asb_package_set_filename (pkg, filename);
 
 	/* call distro-specific method */
 	if (klass->open != NULL)
@@ -788,6 +914,34 @@ asb_package_open (AsbPackage *pkg, const gchar *filename, GError **error)
 	return TRUE;
 }
 
+/**
+ * asb_package_close:
+ * @pkg: A #AsbPackage
+ * @error: A #GError or %NULL
+ *
+ * Closes a package, which can be re-opened if required.
+ *
+ * Returns: %TRUE for success, %FALSE otherwise
+ *
+ * Since: 0.3.5
+ **/
+gboolean
+asb_package_close (AsbPackage *pkg, GError **error)
+{
+	AsbPackageClass *klass = ASB_PACKAGE_GET_CLASS (pkg);
+	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
+
+	/* already closed */
+	if (!priv->is_open)
+		return TRUE;
+	priv->is_open = FALSE;
+
+	/* call distro-specific method */
+	if (klass->close != NULL)
+		return klass->close (pkg, error);
+	return TRUE;
+}
+
 /**
  * asb_package_ensure:
  * @pkg: A #AsbPackage
@@ -808,6 +962,18 @@ asb_package_ensure (AsbPackage *pkg,
 	AsbPackageClass *klass = ASB_PACKAGE_GET_CLASS (pkg);
 	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
 
+	/* reopen as required */
+	if (!priv->is_open) {
+		if (!asb_package_open (pkg, priv->filename, error))
+			return FALSE;
+	}
+
+	/* this is recounted */
+	if (flags & ASB_PACKAGE_ENSURE_DEPS)
+		priv->deps_refcount++;
+	if (flags & ASB_PACKAGE_ENSURE_FILES)
+		priv->filelist_refcount++;
+
 	/* clear flags */
 	if (priv->name != NULL)
 		flags &= ~ASB_PACKAGE_ENSURE_NEVRA;
@@ -821,17 +987,48 @@ asb_package_ensure (AsbPackage *pkg,
 		flags &= ~ASB_PACKAGE_ENSURE_SOURCE;
 	if (priv->filelist != NULL)
 		flags &= ~ASB_PACKAGE_ENSURE_FILES;
-	if (priv->deps != NULL)
+	if (priv->deps->len > 0)
 		flags &= ~ASB_PACKAGE_ENSURE_DEPS;
 	if (priv->releases->len > 0)
 		flags &= ~ASB_PACKAGE_ENSURE_RELEASES;
 
+	/* nothing to do! */
+	if (flags == ASB_PACKAGE_ENSURE_NONE)
+		return TRUE;
+
 	/* call distro-specific method */
 	if (klass->ensure != NULL)
 		return klass->ensure (pkg, flags, error);
 	return TRUE;
 }
 
+/**
+ * asb_package_clear:
+ * @pkg: A #AsbPackage
+ * @flags: #AsbPackageEnsureFlags
+ *
+ * Deallocates previously ensured data.
+ *
+ * Since: 0.3.5
+ **/
+void
+asb_package_clear (AsbPackage *pkg, AsbPackageEnsureFlags flags)
+{
+	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
+
+	/* this is recounted */
+	if (flags & ASB_PACKAGE_ENSURE_DEPS) {
+		if (priv->deps_refcount > 0 && --priv->deps_refcount == 0)
+			g_ptr_array_set_size (priv->deps, 0);
+	}
+	if (flags & ASB_PACKAGE_ENSURE_FILES) {
+		if (priv->filelist_refcount > 0 && --priv->filelist_refcount == 0) {
+			g_strfreev (priv->filelist);
+			priv->filelist = NULL;
+		}
+	}
+}
+
 /**
  * asb_package_explode:
  * @pkg: A #AsbPackage
