@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2014-2015 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -47,7 +47,12 @@ asb_test_get_filename (const gchar *filename)
 	gchar *tmp;
 	_cleanup_free_ gchar *path = NULL;
 
-	path = g_build_filename (TESTDATADIR, filename, NULL);
+	/* try the source then the destdir */
+	path = g_build_filename (TESTDIRSRC, filename, NULL);
+	if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
+		g_free (path);
+		path = g_build_filename (TESTDIRBUILD, filename, NULL);
+	}
 	tmp = realpath (path, full_tmp);
 	if (tmp == NULL)
 		return NULL;
@@ -86,6 +91,7 @@ asb_test_package_rpm_func (void)
 {
 	AsRelease *rel;
 	GError *error = NULL;
+	GPtrArray *deps;
 	GPtrArray *releases;
 	gboolean ret;
 	gchar *tmp;
@@ -129,9 +135,11 @@ asb_test_package_rpm_func (void)
 	g_free (tmp);
 
 	/* deps */
-	tmp = g_strjoinv (";", asb_package_get_deps (pkg));
-	g_assert_cmpstr (tmp, ==, "bar;baz;foo");
-	g_free (tmp);
+	deps = asb_package_get_deps (pkg);
+	g_assert_cmpint (deps->len, ==, 3);
+	g_assert_cmpstr (g_ptr_array_index (deps, 0), ==, "bar");
+	g_assert_cmpstr (g_ptr_array_index (deps, 1), ==, "baz");
+	g_assert_cmpstr (g_ptr_array_index (deps, 2), ==, "foo");
 
 	/* releases */
 	releases = asb_package_get_releases (pkg);
@@ -152,6 +160,29 @@ asb_test_package_rpm_func (void)
 	asb_package_set_config (pkg, "test", "dave2");
 	g_assert_cmpstr (asb_package_get_config (pkg, "test"), ==, "dave2");
 
+	/* clear */
+	asb_package_clear (pkg,
+			   ASB_PACKAGE_ENSURE_DEPS |
+			   ASB_PACKAGE_ENSURE_FILES);
+	g_assert (asb_package_get_filelist (pkg) == NULL);
+	g_assert_cmpint (asb_package_get_deps(pkg)->len, ==, 0);
+
+	/* clear, ensure, ensure, clear, check, clear */
+	asb_package_clear (pkg, ASB_PACKAGE_ENSURE_DEPS);
+	g_assert_cmpint (asb_package_get_deps(pkg)->len, ==, 0);
+	ret = asb_package_ensure (pkg, ASB_PACKAGE_ENSURE_DEPS, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+	g_assert_cmpint (asb_package_get_deps(pkg)->len, ==, 3);
+	ret = asb_package_ensure (pkg, ASB_PACKAGE_ENSURE_DEPS, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+	g_assert_cmpint (asb_package_get_deps(pkg)->len, ==, 3);
+	asb_package_clear (pkg, ASB_PACKAGE_ENSURE_DEPS);
+	g_assert_cmpint (asb_package_get_deps(pkg)->len, ==, 3);
+	asb_package_clear (pkg, ASB_PACKAGE_ENSURE_DEPS);
+	g_assert_cmpint (asb_package_get_deps(pkg)->len, ==, 0);
+
 	/* compare */
 	g_assert_cmpint (asb_package_compare (pkg, pkg), ==, 0);
 
@@ -177,6 +208,18 @@ asb_test_package_rpm_func (void)
 }
 #endif
 
+static void
+asb_test_package_func (void)
+{
+	_cleanup_object_unref_ AsbPackage *pkg = NULL;
+	pkg = asb_package_new ();
+	asb_package_set_filename (pkg, "/tmp/gambit-c-doc-4.7.3-2.fc22.noarch.rpm");
+	g_assert_cmpstr (asb_package_get_nevra (pkg), ==, "gambit-c-doc-4.7.3-2.fc22.noarch");
+	g_assert_cmpstr (asb_package_get_name (pkg), ==, "gambit-c-doc");
+	g_assert_cmpstr (asb_package_get_version (pkg), ==, "4.7.3");
+	g_assert_cmpstr (asb_package_get_arch (pkg), ==, "noarch");
+}
+
 static void
 asb_test_utils_glob_func (void)
 {
@@ -223,18 +266,18 @@ asb_test_plugin_loader_func (void)
 
 	/* get the list of globs */
 	globs = asb_plugin_loader_get_globs (loader);\
-	g_assert_cmpint (globs->len, ==, 22);
+	g_assert_cmpint (globs->len, >=, 44);
 	g_assert_cmpstr (asb_glob_value_search (globs, "/usr/share/applications/gimp.desktop"), ==, "");
+	g_assert_cmpstr (asb_glob_value_search (globs, "/files/share/applications/gimp.desktop"), ==, "");
 	g_assert_cmpstr (asb_glob_value_search (globs, "/srv/dave.txt"), ==, NULL);
 
 	/* get the list of plugins */
 	plugins = asb_plugin_loader_get_plugins (loader);
-	g_assert_cmpint (plugins->len, ==, 17);
+	g_assert_cmpint (plugins->len, >=, 17);
 	plugin = g_ptr_array_index (plugins, 0);
 	g_assert (plugin != NULL);
 	g_assert (plugin->module != NULL);
 	g_assert (plugin->enabled);
-	g_assert_cmpstr (plugin->name, ==, "nm");
 	g_assert (plugin->ctx == ctx);
 
 	/* match the correct one */
@@ -279,21 +322,24 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"composite-1-1.fc21.x86_64.rpm",	/* multiple GUI apps */
 		"font-1-1.fc21.noarch.rpm",		/* font */
 		"font-serif-1-1.fc21.noarch.rpm",	/* font that extends */
+		"colorhug-als-2.0.2.cab",		/* firmware */
 		NULL};
 
 	/* set up the context */
 	ctx = asb_context_new ();
-	g_assert (!asb_context_get_add_cache_id (ctx));
+	g_assert (!asb_context_get_flag (ctx, ASB_CONTEXT_FLAG_ADD_CACHE_ID));
 	asb_context_set_max_threads (ctx, 1);
-	asb_context_set_api_version (ctx, 0.8);
-	asb_context_set_add_cache_id (ctx, TRUE);
-	asb_context_set_no_net (ctx, TRUE);
-	asb_context_set_hidpi_enabled (ctx, TRUE);
+	asb_context_set_api_version (ctx, 0.9);
+	asb_context_set_flags (ctx, ASB_CONTEXT_FLAG_ADD_CACHE_ID |
+				    ASB_CONTEXT_FLAG_NO_NETWORK |
+				    ASB_CONTEXT_FLAG_INCLUDE_FAILED |
+				    ASB_CONTEXT_FLAG_HIDPI_ICONS);
 	asb_context_set_basename (ctx, "appstream");
 	asb_context_set_origin (ctx, "asb-self-test");
 	asb_context_set_cache_dir (ctx, "/tmp/asbuilder/cache");
 	asb_context_set_output_dir (ctx, "/tmp/asbuilder/output");
 	asb_context_set_temp_dir (ctx, "/tmp/asbuilder/temp");
+	asb_context_set_icons_dir (ctx, "/tmp/asbuilder/temp/icons");
 	switch (mode) {
 	case ASB_TEST_CONTEXT_MODE_WITH_CACHE:
 		asb_context_set_old_metadata (ctx, "/tmp/asbuilder/output");
@@ -308,7 +354,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	default:
 		break;
 	}
-	g_assert (asb_context_get_add_cache_id (ctx));
+	g_assert (asb_context_get_flag (ctx, ASB_CONTEXT_FLAG_ADD_CACHE_ID));
 	g_assert_cmpstr (asb_context_get_temp_dir (ctx), ==, "/tmp/asbuilder/temp");
 	ret = asb_context_setup (ctx, &error);
 	g_assert_no_error (error);
@@ -330,7 +376,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	switch (mode) {
 	case ASB_TEST_CONTEXT_MODE_NO_CACHE:
 	case ASB_TEST_CONTEXT_MODE_WITH_OLD_CACHE:
-		g_assert_cmpint (asb_context_get_packages(ctx)->len, ==, 8);
+		g_assert_cmpint (asb_context_get_packages(ctx)->len, ==, 9);
 		break;
 	default:
 		/* no packages should need extracting */
@@ -339,7 +385,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	}
 
 	/* run the plugins */
-	ret = asb_context_process (ctx, AS_CONTEXT_PARSE_FLAG_NONE, &error);
+	ret = asb_context_process (ctx, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
 
@@ -348,6 +394,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	g_assert (g_file_test ("/tmp/asbuilder/output/appstream-failed.xml.gz", G_FILE_TEST_EXISTS));
 	g_assert (g_file_test ("/tmp/asbuilder/output/appstream-ignore.xml.gz", G_FILE_TEST_EXISTS));
 	g_assert (g_file_test ("/tmp/asbuilder/output/appstream-icons.tar.gz", G_FILE_TEST_EXISTS));
+	g_assert (g_file_test ("/tmp/asbuilder/output/appstream-screenshots.tar", G_FILE_TEST_EXISTS));
 
 	/* load AppStream metadata */
 	file = g_file_new_for_path ("/tmp/asbuilder/output/appstream.xml.gz");
@@ -355,7 +402,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	ret = as_store_from_file (store, file, NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
-	g_assert_cmpint (as_store_get_size (store), ==, 5);
+	g_assert_cmpint (as_store_get_size (store), ==, 6);
 	app = as_store_get_app_by_pkgname (store, "app");
 	g_assert (app != NULL);
 	app = as_store_get_app_by_id (store, "app.desktop");
@@ -364,7 +411,33 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	/* check it matches what we expect */
 	xml = as_store_to_xml (store, AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE);
 	expected_xml =
-		"<components version=\"0.8\" builder_id=\"appstream-glib:4\" origin=\"asb-self-test\">\n"
+		"<components version=\"0.9\" builder_id=\"appstream-glib:4\" origin=\"asb-self-test\">\n"
+		"<component type=\"firmware\">\n"
+		"<id>84f40464-9272-4ef7-9399-cd95f12da696</id>\n"
+		"<name>ColorHug Firmware</name>\n"
+		"<summary>Firmware for the ColorHug Colorimeter</summary>\n"
+		"<developer_name>Hughski</developer_name>\n"
+		"<description><p>Updating the firmware on your ColorHug device "
+		"improves performance and adds new features.</p></description>\n"
+		"<icon type=\"stock\">application-x-executable</icon>\n"
+		"<kudos>\n"
+		"<kudo>HiDpiIcon</kudo>\n"
+		"</kudos>\n"
+		"<url type=\"homepage\">http://www.hughski.com/</url>\n"
+		"<releases>\n"
+		"<release version=\"2.0.2\" timestamp=\"1424116753\">\n"
+		"<location>http://www.hughski.com/downloads/colorhug2/firmware/colorhug-2.0.2.cab</location>\n"
+		"<checksum type=\"sha1\">ab2d0294e0d23c1718af79c6a17216c8322ae6da</checksum>\n"
+		"<description><p>This unstable release adds the following features:</p>"
+		"<ul><li>Add TakeReadingArray to enable panel latency measurements</li>"
+		"<li>Speed up the auto-scaled measurements considerably, using 256ms "
+		"as the smallest sample duration</li></ul></description>\n"
+		"</release>\n"
+		"</releases>\n"
+		"<metadata>\n"
+		"<value key=\"X-CacheID\">colorhug-als-2.0.2.cab</value>\n"
+		"</metadata>\n"
+		"</component>\n"
 		"<component type=\"font\">\n"
 		"<id>Liberation</id>\n"
 		"<pkgname>font</pkgname>\n"
@@ -378,11 +451,11 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<screenshots>\n"
 		"<screenshot type=\"default\">\n"
 		"<caption>Liberation Serif – Regular</caption>\n"
-		"<image type=\"source\" height=\"48\" width=\"640\"/>\n"
+		"<image type=\"source\" height=\"48\" width=\"640\">file:/LiberationSerif-85c191549daefb8f3b6cdb4d3d6d8597.png</image>\n"
 		"</screenshot>\n"
 		"<screenshot priority=\"-32\">\n"
 		"<caption>Liberation Serif – Bold</caption>\n"
-		"<image type=\"source\" height=\"48\" width=\"640\"/>\n"
+		"<image type=\"source\" height=\"48\" width=\"640\">file:/LiberationSerif-660e7a5a60fdeb133d3e8b24e2766f2e.png</image>\n"
 		"</screenshot>\n"
 		"</screenshots>\n"
 		"<releases>\n"
@@ -401,6 +474,9 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<source_pkgname>app</source_pkgname>\n"
 		"<name>Extra</name>\n"
 		"<summary>Addons for extra functionality</summary>\n"
+		"<kudos>\n"
+		"<kudo>ModernToolkit</kudo>\n"
+		"</kudos>\n"
 		"<project_license>GPL-2.0+</project_license>\n"
 		"<url type=\"homepage\">http://people.freedesktop.org/</url>\n"
 		"<extends>app.desktop</extends>\n"
@@ -439,6 +515,11 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"</kudos>\n"
 		"<project_license>GPL-2.0+</project_license>\n"
 		"<url type=\"homepage\">http://people.freedesktop.org/~hughsient/appdata/</url>\n"
+		"<screenshots>\n"
+		"<screenshot type=\"default\">\n"
+		"<image type=\"source\">http://people.freedesktop.org/~hughsient/appdata/long-description.png</image>\n"
+		"</screenshot>\n"
+		"</screenshots>\n"
 		"<releases>\n"
 		"<release version=\"1\" timestamp=\"1407844800\"/>\n"
 		"</releases>\n"
@@ -450,7 +531,6 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<lang percentage=\"33\">ru</lang>\n"
 		"</languages>\n"
 		"<metadata>\n"
-		"<value key=\"PaddedIcon\">48x48</value>\n"
 		"<value key=\"X-CacheID\">app-1-1.fc21.x86_64.rpm</value>\n"
 		"</metadata>\n"
 		"</component>\n"
@@ -516,7 +596,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	/* check output */
 	xml_failed = as_store_to_xml (store_failed, AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE);
 	expected_xml =
-		"<components version=\"0.8\" builder_id=\"appstream-glib:4\" origin=\"asb-self-test-failed\">\n"
+		"<components version=\"0.9\" builder_id=\"appstream-glib:4\" origin=\"asb-self-test-failed\">\n"
 		"<component type=\"font\">\n"
 		"<id>LiberationSerif</id>\n"
 		"<pkgname>font-serif</pkgname>\n"
@@ -537,11 +617,11 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<screenshots>\n"
 		"<screenshot type=\"default\">\n"
 		"<caption>Liberation Serif – Regular</caption>\n"
-		"<image type=\"source\" height=\"48\" width=\"640\"/>\n"
+		"<image type=\"source\" height=\"48\" width=\"640\">file:/LiberationSerif-85c191549daefb8f3b6cdb4d3d6d8597.png</image>\n"
 		"</screenshot>\n"
 		"<screenshot priority=\"-32\">\n"
 		"<caption>Liberation Serif – Bold</caption>\n"
-		"<image type=\"source\" height=\"48\" width=\"640\"/>\n"
+		"<image type=\"source\" height=\"48\" width=\"640\">file:/LiberationSerif-660e7a5a60fdeb133d3e8b24e2766f2e.png</image>\n"
 		"</screenshot>\n"
 		"</screenshots>\n"
 		"<releases>\n"
@@ -597,6 +677,9 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<categories>\n"
 		"<category>ConsoleOnly</category>\n"
 		"</categories>\n"
+		"<kudos>\n"
+		"<kudo>ModernToolkit</kudo>\n"
+		"</kudos>\n"
 		"<vetos>\n"
 		"<veto>Required AppData: ConsoleOnly</veto>\n"
 		"</vetos>\n"
@@ -610,7 +693,6 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<lang percentage=\"33\">ru</lang>\n"
 		"</languages>\n"
 		"<metadata>\n"
-		"<value key=\"PaddedIcon\">48x48</value>\n"
 		"<value key=\"X-CacheID\">app-console-1-1.fc21.noarch.rpm</value>\n"
 		"</metadata>\n"
 		"</component>\n"
@@ -624,6 +706,9 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<categories>\n"
 		"<category>ConsoleOnly</category>\n"
 		"</categories>\n"
+		"<kudos>\n"
+		"<kudo>ModernToolkit</kudo>\n"
+		"</kudos>\n"
 		"<vetos>\n"
 		"<veto>Required AppData: ConsoleOnly</veto>\n"
 		"</vetos>\n"
@@ -637,7 +722,6 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<lang percentage=\"33\">ru</lang>\n"
 		"</languages>\n"
 		"<metadata>\n"
-		"<value key=\"PaddedIcon\">48x48</value>\n"
 		"<value key=\"X-CacheID\">app-console-1-1.fc21.noarch.rpm</value>\n"
 		"</metadata>\n"
 		"</component>\n"
@@ -656,7 +740,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	/* check output */
 	xml_ignore = as_store_to_xml (store_ignore, AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE);
 	expected_xml =
-		"<components version=\"0.8\" builder_id=\"appstream-glib:4\" origin=\"asb-self-test-ignore\">\n"
+		"<components version=\"0.9\" builder_id=\"appstream-glib:4\" origin=\"asb-self-test-ignore\">\n"
 		"<component>\n"
 		"<id>app-console.noarch</id>\n"
 		"<metadata>\n"
@@ -752,38 +836,55 @@ asb_test_context_oldcache_func (void)
 }
 
 static void
-asb_test_context_extra_appstream_func (void)
+asb_test_firmware_func (void)
 {
-	GError *error = NULL;
-	gboolean ret;
 	AsApp *app;
 	const gchar *expected_xml;
-	_cleanup_free_ gchar *extra_appdata = NULL;
+	gboolean ret;
+	guint i;
+	_cleanup_error_free_ GError *error = NULL;
 	_cleanup_object_unref_ AsbContext *ctx = NULL;
 	_cleanup_object_unref_ AsStore *store = NULL;
 	_cleanup_object_unref_ GFile *file = NULL;
 	_cleanup_string_free_ GString *xml = NULL;
+	const gchar *filenames[] = {
+		"colorhug-als-2.0.1.cab",
+		"colorhug-als-2.0.0.cab",
+		"colorhug-als-2.0.2.cab",
+		NULL};
 
+	/* set up the context */
 	ctx = asb_context_new ();
-	extra_appdata = asb_test_get_filename ("extra-appdata");
-	g_assert (extra_appdata != NULL);
-	asb_context_set_extra_appstream (ctx, extra_appdata);
 	asb_context_set_max_threads (ctx, 1);
-	asb_context_set_api_version (ctx, 0.8);
-	asb_context_set_add_cache_id (ctx, TRUE);
-	asb_context_set_no_net (ctx, TRUE);
-	asb_context_set_hidpi_enabled (ctx, TRUE);
+	asb_context_set_api_version (ctx, 0.9);
+	asb_context_set_flags (ctx, ASB_CONTEXT_FLAG_NO_NETWORK);
 	asb_context_set_basename (ctx, "appstream");
 	asb_context_set_origin (ctx, "asb-self-test");
 	asb_context_set_cache_dir (ctx, "/tmp/asbuilder/cache");
 	asb_context_set_output_dir (ctx, "/tmp/asbuilder/output");
 	asb_context_set_temp_dir (ctx, "/tmp/asbuilder/temp");
+	asb_context_set_icons_dir (ctx, "/tmp/asbuilder/temp/icons");
 	ret = asb_context_setup (ctx, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
 
+	/* add packages */
+	for (i = 0; filenames[i] != NULL; i++) {
+		_cleanup_free_ gchar *filename = NULL;
+		filename = asb_test_get_filename (filenames[i]);
+		if (filename == NULL)
+			g_warning ("%s not found", filenames[i]);
+		g_assert (filename != NULL);
+		ret = asb_context_add_filename (ctx, filename, &error);
+		g_assert_no_error (error);
+		g_assert (ret);
+	}
+
+	/* verify queue size */
+	g_assert_cmpint (asb_context_get_packages(ctx)->len, ==, 3);
+
 	/* run the plugins */
-	ret = asb_context_process (ctx, AS_CONTEXT_PARSE_FLAG_NONE, &error);
+	ret = asb_context_process (ctx, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
 
@@ -796,38 +897,54 @@ asb_test_context_extra_appstream_func (void)
 	ret = as_store_from_file (store, file, NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
-	g_assert_cmpint (as_store_get_size (store), ==, 2);
-	app = as_store_get_app_by_id (store, "epiphany-test.desktop");
-	g_assert (app != NULL);
-	app = as_store_get_app_by_id (store, "epiphany-local.desktop");
+	g_assert_cmpint (as_store_get_size (store), ==, 1);
+	app = as_store_get_app_by_id (store, "84f40464-9272-4ef7-9399-cd95f12da696");
 	g_assert (app != NULL);
 
 	/* check it matches what we expect */
 	xml = as_store_to_xml (store, AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE);
 	expected_xml =
-		"<components version=\"0.8\" builder_id=\"appstream-glib:4\" origin=\"asb-self-test\">\n"
-		"<component type=\"webapp\">\n"
-		"<id>epiphany-local.desktop</id>\n"
-		"<name>Local</name>\n"
-		"<summary>My local webapp</summary>\n"
-		"<description><p>This is awesome</p></description>\n"
-		"<icon type=\"local\">/usr/share/icons/hicolor/256x256/apps/fedora-logo-icon.png</icon>\n"
-		"<url type=\"homepage\">http://www.hughski.com/</url>\n"
-		"</component>\n"
-		"<component type=\"webapp\">\n"
-		"<id>epiphany-test.desktop</id>\n"
-		"<name>Test</name>\n"
-		"<summary>Please use my awesome webapp</summary>\n"
-		"<description><p>This could be awesome</p></description>\n"
-		"<icon type=\"remote\">http://www.hughski.com/img/logo.png</icon>\n"
+		"<components version=\"0.9\" origin=\"asb-self-test\">\n"
+		"<component type=\"firmware\">\n"
+		"<id>84f40464-9272-4ef7-9399-cd95f12da696</id>\n"
+		"<name>ColorHug Firmware</name>\n"
+		"<summary>Firmware for the ColorHug Colorimeter</summary>\n"
+		"<developer_name>Hughski</developer_name>\n"
+		"<description><p>Updating the firmware on your ColorHug device "
+		"improves performance and adds new features.</p></description>\n"
+		"<icon type=\"stock\">application-x-executable</icon>\n"
+		"<kudos>\n"
+		"<kudo>HiDpiIcon</kudo>\n"
+		"</kudos>\n"
 		"<url type=\"homepage\">http://www.hughski.com/</url>\n"
+		"<releases>\n"
+		"<release version=\"2.0.2\" timestamp=\"1424116753\">\n"
+		"<location>http://www.hughski.com/downloads/colorhug2/firmware/colorhug-2.0.2.cab</location>\n"
+		"<checksum type=\"sha1\">ab2d0294e0d23c1718af79c6a17216c8322ae6da</checksum>\n"
+		"<description><p>This unstable release adds the following features:</p>"
+		"<ul><li>Add TakeReadingArray to enable panel latency measurements</li>"
+		"<li>Speed up the auto-scaled measurements considerably, using 256ms "
+		"as the smallest sample duration</li></ul></description>\n"
+		"</release>\n"
+		"<release version=\"2.0.1\" timestamp=\"1424116753\">\n"
+		"<location>http://www.hughski.com/downloads/colorhug2/firmware/colorhug-2.0.1.cab</location>\n"
+		"<checksum type=\"sha1\">cc07f12dfea7103774decbcf48c47cc0604fbc4e</checksum>\n"
+		"<description><p>This unstable release adds the following features:</p>"
+		"<ul><li>Use TakeReadings() to do a quick non-adaptive measurement</li>"
+		"<li>Scale XYZ measurement with a constant factor to make the CCMX more "
+		"sane</li></ul></description>\n"
+		"</release>\n"
+		"<release version=\"2.0.0\" timestamp=\"1425168000\">\n"
+		"<location>http://www.hughski.com/downloads/colorhug2/firmware/colorhug-2.0.0.cab</location>\n"
+		"<checksum type=\"sha1\">81431b6d3c9f90c3137a8a132d4fb5e3681e6791</checksum>\n"
+		"</release>\n"
+		"</releases>\n"
 		"</component>\n"
 		"</components>\n";
 	ret = asb_test_compare_lines (xml->str, expected_xml, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
 
-	/* remove temp space */
 	ret = asb_utils_rmtree ("/tmp/asbuilder", &error);
 	g_assert_no_error (error);
 	g_assert (ret);
@@ -844,13 +961,14 @@ main (int argc, char **argv)
 	g_setenv ("ASB_IS_SELF_TEST", "", TRUE);
 
 	/* tests go here */
+	g_test_add_func ("/AppStreamBuilder/package", asb_test_package_func);
 	g_test_add_func ("/AppStreamBuilder/utils{replace}", asb_test_utils_replace_func);
 	g_test_add_func ("/AppStreamBuilder/utils{glob}", asb_test_utils_glob_func);
 	g_test_add_func ("/AppStreamBuilder/plugin-loader", asb_test_plugin_loader_func);
+	g_test_add_func ("/AppStreamBuilder/firmware", asb_test_firmware_func);
 	g_test_add_func ("/AppStreamBuilder/context{no-cache}", asb_test_context_nocache_func);
 	g_test_add_func ("/AppStreamBuilder/context{cache}", asb_test_context_cache_func);
 	g_test_add_func ("/AppStreamBuilder/context{old-cache}", asb_test_context_oldcache_func);
-	g_test_add_func ("/AppStreamBuilder/context{extra-appstream}", asb_test_context_extra_appstream_func);
 #ifdef HAVE_RPM
 	g_test_add_func ("/AppStreamBuilder/package{rpm}", asb_test_package_rpm_func);
 #endif
