@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2014-2015 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -43,11 +43,9 @@ struct _AsbTaskPrivate
 {
 	AsbContext		*ctx;
 	AsbPackage		*pkg;
-	AsbPanel		*panel;
 	GPtrArray		*plugins_to_run;
 	gchar			*filename;
 	gchar			*tmpdir;
-	guint			 id;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsbTask, asb_task, G_TYPE_OBJECT)
@@ -99,35 +97,55 @@ asb_task_explode_extra_package (AsbTask *task,
 {
 	AsbTaskPrivate *priv = GET_PRIVATE (task);
 	AsbPackage *pkg_extra;
-	gboolean ret = TRUE;
+	GPtrArray *deps;
+	guint i;
+	const gchar *dep;
 
 	/* if not found, that's fine */
 	pkg_extra = asb_context_find_by_pkgname (priv->ctx, pkg_name);
 	if (pkg_extra == NULL)
 		return TRUE;
+
+	/* check it's from the same source package */
 	if (!asb_package_ensure (pkg_extra,
-				 ASB_PACKAGE_ENSURE_FILES |
 				 ASB_PACKAGE_ENSURE_SOURCE,
 				 error))
 		return FALSE;
-
-	/* check it's from the same source package */
 	if (require_same_srpm &&
 	    (g_strcmp0 (asb_package_get_source (pkg_extra),
 		        asb_package_get_source (priv->pkg)) != 0))
 		return TRUE;
-
-	asb_panel_set_status (priv->panel, "Decompressing extra pkg %s",
-			      asb_package_get_name (pkg_extra));
+	g_debug ("decompressing extra pkg %s", asb_package_get_name (pkg_extra));
 	asb_package_log (priv->pkg,
 			 ASB_PACKAGE_LOG_LEVEL_DEBUG,
 			 "Adding extra package %s for %s",
 			 asb_package_get_name (pkg_extra),
 			 asb_package_get_name (priv->pkg));
-	ret = asb_package_explode (pkg_extra, priv->tmpdir,
-				   asb_context_get_file_globs (priv->ctx),
-				   error);
-	return ret;
+	if (!asb_package_ensure (pkg_extra,
+				 ASB_PACKAGE_ENSURE_FILES |
+				 ASB_PACKAGE_ENSURE_DEPS,
+				 error))
+		return FALSE;
+	if (!asb_package_explode (pkg_extra, priv->tmpdir,
+				  asb_context_get_file_globs (priv->ctx),
+				  error))
+		return FALSE;
+
+	/* copy all the extra package requires into the main package too */
+	deps = asb_package_get_deps (pkg_extra);
+	for (i = 0; i < deps->len; i++) {
+		dep = g_ptr_array_index (deps, i);
+		asb_package_add_dep (priv->pkg, dep);
+	}
+
+	/* free resources */
+	if (!asb_package_close (pkg_extra, error))
+		return FALSE;
+	asb_package_clear (pkg_extra,
+			   ASB_PACKAGE_ENSURE_DEPS |
+			   ASB_PACKAGE_ENSURE_FILES);
+
+	return TRUE;
 }
 
 /**
@@ -137,45 +155,48 @@ static gboolean
 asb_task_explode_extra_packages (AsbTask *task, GError **error)
 {
 	AsbTaskPrivate *priv = GET_PRIVATE (task);
+	GPtrArray *deps;
 	const gchar *ignore[] = { "rtld", NULL };
 	const gchar *tmp;
-	gchar **deps;
 	guint i;
 	_cleanup_hashtable_unref_ GHashTable *hash = NULL;
 	_cleanup_ptrarray_unref_ GPtrArray *array = NULL;
 	_cleanup_ptrarray_unref_ GPtrArray *icon_themes = NULL;
 
 	/* anything the package requires */
-	hash = g_hash_table_new (g_str_hash, g_str_equal);
+	hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 	for (i = 0; ignore[i] != NULL; i++) {
 		g_hash_table_insert (hash,
-				     (gchar *) ignore[i],
+				     g_strdup (ignore[i]),
 				     GINT_TO_POINTER (1));
 	}
 	array = g_ptr_array_new_with_free_func (g_free);
 	icon_themes = g_ptr_array_new_with_free_func (g_free);
 	deps = asb_package_get_deps (priv->pkg);
-	for (i = 0; deps[i] != NULL; i++) {
-		if (g_strstr_len (deps[i], -1, " ") != NULL)
+	for (i = 0; i < deps->len; i++) {
+		tmp = g_ptr_array_index (deps, i);
+		if (g_strstr_len (tmp, -1, " ") != NULL)
 			continue;
-		if (g_strstr_len (deps[i], -1, ".so") != NULL)
+		if (g_strstr_len (tmp, -1, ".so") != NULL)
 			continue;
-		if (g_str_has_prefix (deps[i], "/"))
+		if (g_str_has_prefix (tmp, "/"))
 			continue;
-		if (g_hash_table_lookup (hash, deps[i]) != NULL)
+		if (g_hash_table_lookup (hash, tmp) != NULL)
+			continue;
+		if (g_strcmp0 (tmp, asb_package_get_name (priv->pkg)) == 0)
 			continue;
 		/* if an app depends on kde-runtime, that means the
 		 * oxygen icon set is available to them */
-		if (g_strcmp0 (deps[i], "oxygen-icon-theme") == 0 ||
-		    g_strcmp0 (deps[i], "kde-runtime") == 0) {
-			g_hash_table_insert (hash, (gpointer) "oxygen-icon-theme",
+		if (g_strcmp0 (tmp, "oxygen-icon-theme") == 0 ||
+		    g_strcmp0 (tmp, "kde-runtime") == 0) {
+			g_hash_table_insert (hash, g_strdup ("oxygen-icon-theme"),
 					     GINT_TO_POINTER (1));
 			g_ptr_array_add (icon_themes,
 					 g_strdup ("oxygen-icon-theme"));
 		} else {
-			g_ptr_array_add (array, g_strdup (deps[i]));
+			g_ptr_array_add (array, g_strdup (tmp));
 		}
-		g_hash_table_insert (hash, deps[i], GINT_TO_POINTER (1));
+		g_hash_table_insert (hash, g_strdup (tmp), GINT_TO_POINTER (1));
 	}
 
 	/* explode any potential packages */
@@ -232,9 +253,7 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 				 error_not_used))
 		return FALSE;
 
-	asb_panel_set_job_number (priv->panel, priv->id + 1);
-	asb_panel_set_title (priv->panel, asb_package_get_name (priv->pkg));
-	asb_panel_set_status (priv->panel, "Starting");
+	g_debug ("starting: %s", asb_package_get_name (priv->pkg));
 
 	/* ensure file list read */
 	if (!asb_package_ensure (priv->pkg,
@@ -264,7 +283,7 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 	}
 
 	/* explode tree */
-	asb_panel_set_status (priv->panel, "Decompressing files");
+	g_debug ("decompressing files: %s", asb_package_get_name (priv->pkg));
 	asb_package_log (priv->pkg,
 			 ASB_PACKAGE_LOG_LEVEL_DEBUG,
 			 "Exploding tree for %s",
@@ -297,7 +316,7 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 	}
 
 	/* run plugins */
-	asb_panel_set_status (priv->panel, "Examining");
+	g_debug ("examining: %s", asb_package_get_name (priv->pkg));
 	for (i = 0; i < priv->plugins_to_run->len; i++) {
 		GList *apps_tmp = NULL;
 		plugin = g_ptr_array_index (priv->plugins_to_run, i);
@@ -324,7 +343,7 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 		goto skip;
 
 	/* print */
-	asb_panel_set_status (priv->panel, "Processing");
+	g_debug ("processing: %s", asb_package_get_name (priv->pkg));
 	for (l = apps; l != NULL; l = l->next) {
 		app = l->data;
 
@@ -399,7 +418,7 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 		}
 
 		/* set cache-id in case we want to use the metadata directly */
-		if (asb_context_get_add_cache_id (priv->ctx)) {
+		if (asb_context_get_flag (priv->ctx, ASB_CONTEXT_FLAG_ADD_CACHE_ID)) {
 			cache_id = asb_utils_get_cache_id_for_filename (priv->filename);
 			as_app_add_metadata (AS_APP (app),
 					     "X-CacheID",
@@ -429,11 +448,11 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 skip:
 	/* add a dummy element to the AppStream metadata so that we don't keep
 	 * parsing this every time */
-	if (asb_context_get_add_cache_id (priv->ctx) && nr_added == 0)
+	if (asb_context_get_flag (priv->ctx, ASB_CONTEXT_FLAG_ADD_CACHE_ID) && nr_added == 0)
 		asb_context_add_app_ignore (priv->ctx, priv->pkg);
 
 	/* delete tree */
-	asb_panel_set_status (priv->panel, "Deleting temp files");
+	g_debug ("deleting temp files: %s", asb_package_get_name (priv->pkg));
 	if (!asb_utils_rmtree (priv->tmpdir, &error)) {
 		asb_package_log (priv->pkg,
 				 ASB_PACKAGE_LOG_LEVEL_WARNING,
@@ -443,7 +462,7 @@ skip:
 	}
 
 	/* write log */
-	asb_panel_set_status (priv->panel, "Writing log");
+	g_debug ("writing log: %s", asb_package_get_name (priv->pkg));
 	if (!asb_package_log_flush (priv->pkg, &error)) {
 		asb_package_log (priv->pkg,
 				 ASB_PACKAGE_LOG_LEVEL_WARNING,
@@ -451,10 +470,12 @@ skip:
 				 error->message);
 		goto out;
 	}
-
-	/* update UI */
-	asb_panel_remove (priv->panel);
 out:
+	/* clear loaded resources */
+	asb_package_close (priv->pkg, NULL);
+	asb_package_clear (priv->pkg,
+			   ASB_PACKAGE_ENSURE_DEPS |
+			   ASB_PACKAGE_ENSURE_FILES);
 	g_list_free_full (apps, (GDestroyNotify) g_object_unref);
 	return TRUE;
 }
@@ -472,8 +493,6 @@ asb_task_finalize (GObject *object)
 	g_ptr_array_unref (priv->plugins_to_run);
 	if (priv->pkg != NULL)
 		g_object_unref (priv->pkg);
-	if (priv->panel != NULL)
-		g_object_unref (priv->panel);
 	g_free (priv->filename);
 	g_free (priv->tmpdir);
 
@@ -499,38 +518,6 @@ asb_task_set_package (AsbTask *task, AsbPackage *pkg)
 	priv->pkg = g_object_ref (pkg);
 }
 
-/**
- * asb_task_set_panel: (skip):
- * @task: A #AsbTask
- * @panel: A #AsbPanel
- *
- * Sets the panel used for the task.
- *
- * Since: 0.2.3
- **/
-void
-asb_task_set_panel (AsbTask *task, AsbPanel *panel)
-{
-	AsbTaskPrivate *priv = GET_PRIVATE (task);
-	priv->panel = g_object_ref (panel);
-}
-
-/**
- * asb_task_set_id:
- * @task: A #AsbTask
- * @id: numeric identifier
- *
- * Sets the ID to use for the task.
- *
- * Since: 0.1.0
- **/
-void
-asb_task_set_id (AsbTask *task, guint id)
-{
-	AsbTaskPrivate *priv = GET_PRIVATE (task);
-	priv->id = id;
-}
-
 /**
  * asb_task_init:
  **/
