@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2014-2015 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -22,16 +22,9 @@
 #include <config.h>
 
 #include <appstream-glib.h>
-#include <libsoup/soup.h>
 
 #include <asb-plugin.h>
 
-struct AsbPluginPrivate {
-	SoupSession	*session;
-	GPtrArray	*filenames;
-	GMutex		 filenames_mutex;
-};
-
 /**
  * asb_plugin_get_name:
  */
@@ -41,43 +34,6 @@ asb_plugin_get_name (void)
 	return "appdata";
 }
 
-/**
- * asb_plugin_initialize:
- */
-void
-asb_plugin_initialize (AsbPlugin *plugin)
-{
-	plugin->priv = ASB_PLUGIN_GET_PRIVATE (AsbPluginPrivate);
-	plugin->priv->filenames = g_ptr_array_new_with_free_func (g_free);
-	g_mutex_init (&plugin->priv->filenames_mutex);
-	plugin->priv->session = soup_session_new_with_options (SOUP_SESSION_USER_AGENT, "createrepo_as",
-							       SOUP_SESSION_TIMEOUT, 10,
-							       NULL);
-	soup_session_add_feature_by_type (plugin->priv->session,
-					  SOUP_TYPE_PROXY_RESOLVER_DEFAULT);
-}
-
-/**
- * asb_plugin_destroy:
- */
-void
-asb_plugin_destroy (AsbPlugin *plugin)
-{
-	const gchar *tmp;
-	guint i;
-
-	/* print out AppData files not used */
-	if (g_getenv ("ASB_PERFORM_EXTRA_CHECKS") != NULL) {
-		for (i = 0; i < plugin->priv->filenames->len; i++) {
-			tmp = g_ptr_array_index (plugin->priv->filenames, i);
-			g_debug ("%s was not used", tmp);
-		}
-	}
-	g_ptr_array_unref (plugin->priv->filenames);
-	g_mutex_clear (&plugin->priv->filenames_mutex);
-	g_object_unref (plugin->priv->session);
-}
-
 /**
  * asb_plugin_add_globs:
  */
@@ -85,75 +41,11 @@ void
 asb_plugin_add_globs (AsbPlugin *plugin, GPtrArray *globs)
 {
 	asb_plugin_add_glob (globs, "/usr/share/appdata/*.appdata.xml");
+	asb_plugin_add_glob (globs, "*.metainfo.xml");
 }
 
 /**
- * asb_plugin_appdata_add_path:
- */
-static gboolean
-asb_plugin_appdata_add_path (AsbPlugin *plugin, const gchar *path, GError **error)
-{
-	const gchar *tmp;
-	_cleanup_dir_close_ GDir *dir = NULL;
-
-	/* scan all the files */
-	dir = g_dir_open (path, 0, error);
-	if (dir == NULL)
-		return FALSE;
-	while ((tmp = g_dir_read_name (dir)) != NULL) {
-		_cleanup_free_ gchar *filename = NULL;
-		filename = g_build_filename (path, tmp, NULL);
-		if (g_file_test (filename, G_FILE_TEST_IS_DIR)) {
-			if (!asb_plugin_appdata_add_path (plugin, filename, error))
-				return FALSE;
-		} else {
-			g_ptr_array_add (plugin->priv->filenames, g_strdup (filename));
-		}
-	}
-	return TRUE;
-}
-
-/**
- * asb_plugin_appdata_add_files:
- */
-static gboolean
-asb_plugin_appdata_add_files (AsbPlugin *plugin, const gchar *path, GError **error)
-{
-	gboolean ret;
-
-	/* already done */
-	if (plugin->priv->filenames->len > 0)
-		return TRUE;
-
-	g_mutex_lock (&plugin->priv->filenames_mutex);
-	ret = asb_plugin_appdata_add_path (plugin, path, error);
-	g_mutex_unlock  (&plugin->priv->filenames_mutex);
-	return ret;
-}
-
-/**
- * asb_plugin_appdata_remove_file:
- */
-static void
-asb_plugin_appdata_remove_file (AsbPlugin *plugin, const gchar *filename)
-{
-	const gchar *tmp;
-	guint i;
-
-	g_mutex_lock (&plugin->priv->filenames_mutex);
-	for (i = 0; i < plugin->priv->filenames->len; i++) {
-		tmp = g_ptr_array_index (plugin->priv->filenames, i);
-		if (g_strcmp0 (tmp, filename) == 0) {
-			g_ptr_array_remove_fast (plugin->priv->filenames,
-						 (gpointer) tmp);
-			break;
-		}
-	}
-	g_mutex_unlock  (&plugin->priv->filenames_mutex);
-}
-
-/**
- * asb_plugin_process_filename:
+ * asb_plugin_appdata_log_overwrite:
  */
 static void
 asb_plugin_appdata_log_overwrite (AsbApp *app,
@@ -174,80 +66,6 @@ asb_plugin_appdata_log_overwrite (AsbApp *app,
 	}
 }
 
-/**
- * asb_plugin_appdata_load_url:
- **/
-static gboolean
-asb_plugin_appdata_load_url (AsbPlugin *plugin,
-			     AsbApp *app,
-			     const gchar *url,
-			     GError **error)
-{
-	const gchar *cache_dir;
-	gboolean ret = TRUE;
-	SoupStatus status;
-	SoupURI *uri = NULL;
-	_cleanup_free_ gchar *basename = NULL;
-	_cleanup_free_ gchar *cache_filename = NULL;
-	_cleanup_object_unref_ SoupMessage *msg = NULL;
-
-	/* download to cache if not already added */
-	basename = g_path_get_basename (url);
-	cache_dir = asb_package_get_config (asb_app_get_package (app), "CacheDir");
-	cache_filename = g_strdup_printf ("%s/%s-%s",
-					  cache_dir,
-					  as_app_get_id_filename (AS_APP (app)),
-					  basename);
-	if (!g_file_test (cache_filename, G_FILE_TEST_EXISTS)) {
-		if (asb_context_get_no_net (plugin->ctx)) {
-			asb_package_log (asb_app_get_package (app),
-					 ASB_PACKAGE_LOG_LEVEL_WARNING,
-					 "Could not download %s as no network", url);
-			goto out;
-		}
-		uri = soup_uri_new (url);
-		if (uri == NULL) {
-			ret = FALSE;
-			g_set_error (error,
-				     ASB_PLUGIN_ERROR,
-				     ASB_PLUGIN_ERROR_FAILED,
-				     "Could not parse '%s' as a URL", url);
-			goto out;
-		}
-		asb_package_log (asb_app_get_package (app),
-				 ASB_PACKAGE_LOG_LEVEL_DEBUG,
-				 "Downloading %s", url);
-		msg = soup_message_new_from_uri (SOUP_METHOD_GET, uri);
-		status = soup_session_send_message (plugin->priv->session, msg);
-		if (status != SOUP_STATUS_OK) {
-			ret = FALSE;
-			g_set_error (error,
-				     ASB_PLUGIN_ERROR,
-				     ASB_PLUGIN_ERROR_FAILED,
-				     "Downloading failed: %s",
-				     soup_status_get_phrase (status));
-			goto out;
-		}
-
-		/* save new file */
-		ret = g_file_set_contents (cache_filename,
-					   msg->response_body->data,
-					   msg->response_body->length,
-					   error);
-		if (!ret)
-			goto out;
-	}
-
-	/* load the pixbuf */
-	ret = asb_app_add_screenshot_source (app, cache_filename, error);
-	if (!ret)
-		goto out;
-out:
-	if (uri != NULL)
-		soup_uri_free (uri);
-	return ret;
-}
-
 /**
  * asb_plugin_process_filename:
  */
@@ -414,34 +232,12 @@ asb_plugin_process_filename (AsbPlugin *plugin,
 	/* add screenshots if not already added */
 	array = as_app_get_screenshots (AS_APP (app));
 	if (array->len == 0) {
+		/* just use the upstream locations */
 		array = as_app_get_screenshots (appdata);
 		for (i = 0; i < array->len; i++) {
-			GError *error_local = NULL;
-			AsScreenshot *ass;
-			AsImage *image;
-
-			ass = g_ptr_array_index (array, i);
-			image = as_screenshot_get_source (ass);
-			if (image == NULL)
-				continue;
-
-			/* load the URI or get from a cache */
-			tmp = as_image_get_url (image);
-			ret = asb_plugin_appdata_load_url (plugin,
-							   app,
-							   as_image_get_url (image),
-							   &error_local);
-			if (ret) {
-				asb_package_log (asb_app_get_package (app),
-						 ASB_PACKAGE_LOG_LEVEL_DEBUG,
-						 "Added screenshot %s", tmp);
-			} else {
-				asb_package_log (asb_app_get_package (app),
-						 ASB_PACKAGE_LOG_LEVEL_WARNING,
-						 "Failed to load screenshot %s: %s",
-						 tmp, error_local->message);
-				g_clear_error (&error_local);
-			}
+			AsScreenshot *ss;
+			ss = g_ptr_array_index (array, i);
+			as_app_add_screenshot (AS_APP (app), ss);
 		}
 	} else {
 		array = as_app_get_screenshots (appdata);
@@ -463,6 +259,13 @@ asb_plugin_process_filename (AsbPlugin *plugin,
 		as_app_add_metadata (AS_APP (app), key, tmp, -1);
 	}
 
+	/* add releases */
+	array = as_app_get_releases (appdata);
+	for (i = 0; i < array->len; i++) {
+		AsRelease *rel = g_ptr_array_index (array, i);
+		as_app_add_release (AS_APP (app), rel);
+	}
+
 	/* log updateinfo */
 	tmp = as_app_get_update_contact (AS_APP (appdata));
 	if (tmp != NULL) {
@@ -503,45 +306,28 @@ asb_plugin_process_app (AsbPlugin *plugin,
 			GError **error)
 {
 	GError *error_local = NULL;
-	const gchar *kind_str;
-	const gchar *tmp;
-	_cleanup_free_ gchar *appdata_basename = NULL;
 	_cleanup_free_ gchar *appdata_filename = NULL;
-	_cleanup_free_ gchar *appdata_filename_extra = NULL;
 
 	/* get possible sources */
-	appdata_basename = asb_plugin_appdata_get_fn_for_app (AS_APP (app));
-	appdata_filename = g_strdup_printf ("%s/usr/share/appdata/%s.appdata.xml",
-					    tmpdir, appdata_basename);
-	tmp = asb_package_get_config (pkg, "AppDataExtra");
-	if (tmp != NULL && g_file_test (tmp, G_FILE_TEST_EXISTS)) {
-		if (!asb_plugin_appdata_add_files (plugin, tmp, error))
-			return FALSE;
-		kind_str = as_id_kind_to_string (as_app_get_id_kind (AS_APP (app)));
-		appdata_filename_extra = g_strdup_printf ("%s/%s/%s.appdata.xml",
-							  tmp,
-							  kind_str,
-							  appdata_basename);
-		if (g_file_test (appdata_filename, G_FILE_TEST_EXISTS) &&
-		    g_file_test (appdata_filename_extra, G_FILE_TEST_EXISTS)) {
-			asb_package_log (pkg,
-					 ASB_PACKAGE_LOG_LEVEL_WARNING,
-					 "extra AppData file %s overwrites upstream",
-					 appdata_filename_extra);
+	if (asb_package_get_kind (pkg) == ASB_PACKAGE_KIND_FIRMWARE) {
+		appdata_filename = g_strdup_printf ("%s/%s.metainfo.xml",
+						    tmpdir,
+						    asb_package_get_source_pkgname (pkg));
+	} else {
+		_cleanup_free_ gchar *appdata_basename = NULL;
+		appdata_basename = asb_plugin_appdata_get_fn_for_app (AS_APP (app));
+		appdata_filename = g_strdup_printf ("%s/files/share/appdata/%s.appdata.xml",
+						    tmpdir, appdata_basename);
+		if (!g_file_test (appdata_filename, G_FILE_TEST_EXISTS)) {
+			g_free (appdata_filename);
+			appdata_filename = g_strdup_printf ("%s/usr/share/appdata/%s.appdata.xml",
+							    tmpdir, appdata_basename);
+		}
+		if (!g_file_test (appdata_filename, G_FILE_TEST_EXISTS)) {
+			g_free (appdata_filename);
+			appdata_filename = g_strdup_printf ("%s/usr/share/appdata-extra/%s.appdata.xml",
+							    tmpdir, appdata_basename);
 		}
-
-		/* we used this */
-		asb_plugin_appdata_remove_file (plugin, appdata_filename_extra);
-	}
-
-	/* any appdata-extra file */
-	if (appdata_filename_extra != NULL &&
-	    g_file_test (appdata_filename_extra, G_FILE_TEST_EXISTS)) {
-		as_app_add_metadata (AS_APP (app), "DistroMetadata", NULL, -1);
-		return asb_plugin_process_filename (plugin,
-						    app,
-						    appdata_filename_extra,
-						    error);
 	}
 
 	/* any installed appdata file */
