@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2014-2015 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -58,9 +58,9 @@ asb_plugin_add_globs (AsbPlugin *plugin, GPtrArray *globs)
 static gboolean
 _asb_plugin_check_filename (const gchar *filename)
 {
-	if (fnmatch ("/usr/share/applications/*.desktop", filename, 0) == 0)
+	if (asb_plugin_match_glob ("/usr/share/applications/*.desktop", filename))
 		return TRUE;
-	if (fnmatch ("/usr/share/applications/kde4/*.desktop", filename, 0) == 0)
+	if (asb_plugin_match_glob ("/usr/share/applications/kde4/*.desktop", filename))
 		return TRUE;
 	return FALSE;
 }
@@ -138,8 +138,6 @@ asb_app_load_icon (AsbApp *app,
 		size_str = g_strdup_printf ("%ix%i",
 					    pixbuf_width,
 					    pixbuf_height);
-		as_app_add_metadata (AS_APP (app), "PaddedIcon",
-				     size_str, -1);
 		asb_package_log (asb_app_get_package (app),
 				 ASB_PACKAGE_LOG_LEVEL_INFO,
 				 "icon %s padded to %ix%i as size %s",
@@ -250,7 +248,7 @@ asb_plugin_desktop_add_icons (AsbPlugin *plugin,
 	}
 
 	/* save in target directory */
-	if (asb_context_get_hidpi_enabled (plugin->ctx)) {
+	if (asb_context_get_flag (plugin->ctx, ASB_CONTEXT_FLAG_HIDPI_ICONS)) {
 		name = g_strdup_printf ("%ix%i/%s.png",
 					64, 64,
 					as_app_get_id_filename (AS_APP (app)));
@@ -266,7 +264,7 @@ asb_plugin_desktop_add_icons (AsbPlugin *plugin,
 	as_app_add_icon (AS_APP (app), icon);
 
 	/* is HiDPI disabled */
-	if (!asb_context_get_hidpi_enabled (plugin->ctx))
+	if (!asb_context_get_flag (plugin->ctx, ASB_CONTEXT_FLAG_HIDPI_ICONS))
 		return TRUE;
 
 	/* try to get a HiDPI icon */
@@ -321,7 +319,7 @@ asb_plugin_process_filename (AsbPlugin *plugin,
 	/* create app */
 	app_id = g_path_get_basename (filename);
 	app = asb_app_new (pkg, app_id);
-	asb_app_set_hidpi_enabled (app, asb_context_get_hidpi_enabled (plugin->ctx));
+	asb_app_set_hidpi_enabled (app, asb_context_get_flag (plugin->ctx, ASB_CONTEXT_FLAG_HIDPI_ICONS));
 	full_filename = g_build_filename (tmpdir, filename, NULL);
 	ret = as_app_parse_file (AS_APP (app),
 				 full_filename,
