@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2014-2015 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -63,9 +63,9 @@ asb_plugin_add_globs (AsbPlugin *plugin, GPtrArray *globs)
 static gboolean
 _asb_plugin_check_filename (const gchar *filename)
 {
-	if (fnmatch ("/usr/share/fonts/*/*.otf", filename, 0) == 0)
+	if (asb_plugin_match_glob ("/usr/share/fonts/*/*.otf", filename))
 		return TRUE;
-	if (fnmatch ("/usr/share/fonts/*/*.ttf", filename, 0) == 0)
+	if (asb_plugin_match_glob ("/usr/share/fonts/*/*.ttf", filename))
 		return TRUE;
 	return FALSE;
 }
@@ -90,6 +90,8 @@ asb_font_fix_metadata (AsbApp *app)
 	} text_icon[] =  {
 		{ "en",		"Aa" },
 		{ "ar",		"أب" },
+		{ "as",         "অআই" },
+		{ "bn",         "অআই" },
 		{ "be",		"Аа" },
 		{ "bg",		"Аа" },
 		{ "cs",		"Aa" },
@@ -98,14 +100,21 @@ asb_font_fix_metadata (AsbApp *app)
 		{ "es",		"Aa" },
 		{ "fr",		"Aa" },
 		{ "gu",		"અબક" },
+		{ "hi",         "अआइ" },
 		{ "he",		"אב" },
 		{ "it",		"Aa" },
+		{ "kn",         "ಅಆಇ" },
 		{ "ml",		"ആഇ" },
+		{ "ne",         "अआइ" },
 		{ "nl",		"Aa" },
+		{ "or",         "ଅଆଇ" },
+		{ "pa",         "ਅਆਇ" },
 		{ "pl",		"ĄĘ" },
 		{ "pt",		"Aa" },
 		{ "ru",		"Аа" },
 		{ "sv",		"Åäö" },
+		{ "ta",         "அஆஇ" },
+		{ "te",         "అఆఇ" },
 		{ "ua",		"Аа" },
 		{ "zh-tw",	"漢" },
 		{ NULL, NULL } };
@@ -115,6 +124,8 @@ asb_font_fix_metadata (AsbApp *app)
 	} text_sample[] =  {
 		{ "en",		"How quickly daft jumping zebras vex." },
 		{ "ar",		"نصٌّ حكيمٌ لهُ سِرٌّ قاطِعٌ وَذُو شَأنٍ عَظيمٍ مكتوبٌ على ثوبٍ أخضرَ ومُغلفٌ بجلدٍ أزرق" },
+		{ "as",		"আর আপনি সাক্ষাৎ"},
+		{ "bn",		"আর আপনি সাক্ষাৎ"},
 		{ "be",		"У Іўі худы жвавы чорт у зялёнай камізэльцы пабег пад’есці фаршу з юшкай." },
 		{ "bg",		"Под южно дърво, цъфтящо в синьо, бягаше малко, пухкаво зайче." },
 		{ "cs",		"Příliš žluťoučký kůň úpěl ďábelské ódy" },
@@ -124,14 +135,21 @@ asb_font_fix_metadata (AsbApp *app)
 		{ "fr",		"Voix ambiguë d'un cœur qui, au zéphyr, préfère les jattes de kiwis." },
 		{ "gu",		"ઇ.સ. ૧૯૭૮ ની ૨૫ તારીખે, ૦૬-૩૪ વાગે, ઐશ્વર્યવાન, વફાદાર, અંગ્રેજ ઘરધણીના આ ઝાડ પાસે ઊભેલા બાદશાહ; અને ઓસરીમાંના ઠળીયા તથા છાણાના ઢગલા દુર કરીને, ઔપચારીકતાથી ઉભેલા ઋષી સમાન પ્રજ્ઞાચક્ષુ ખાલસાજી ભટ મળ્યા હતા." },
 		{ "he",		"יַעֲקֹב בֶּן־דָּגָן הַשָּׂמֵחַ טִפֵּס בֶּעֱזוּז לְרֹאשׁ סֻלָּם מָאֳרָךְ לִצְפּוֹת בִּמְעוֹף דּוּכִיפַת וְנֵץ" },
+		{ "hi",		"आपसे मिलकर खुशी हुई "},
 		{ "it",		"Senza qualche prova ho il dubbio che si finga morto." },
 		{ "ja",		"いろはにほへと ちりぬるを わかよたれそ つねならむ うゐのおくやま けふこえて あさきゆめみし ゑひもせす" },
+		{ "kn",		"ಸಂತೋಷ ನೀವು ಭೇಟಿ"},
 		{ "ml",		"അജവും ആനയും ഐരാവതവും ഗരുഡനും കഠോര സ്വരം പൊഴിക്കെ ഹാരവും ഒഢ്യാണവും ഫാലത്തില്‍ മഞ്ഞളും ഈറന്‍ കേശത്തില്‍ ഔഷധ എണ്ണയുമായി ഋതുമതിയും അനഘയും ഭൂനാഥയുമായ ഉമ ദുഃഖഛവിയോടെ ഇടതു പാദം ഏന്തി ങ്യേയാദൃശം നിര്‍ഝരിയിലെ ചിറ്റലകളെ ഓമനിക്കുമ്പോള്‍ ബാ‍ലയുടെ കണ്‍കളില്‍ നീര്‍ ഊര്‍ന്നു വിങ്ങി" },
+		{ "ne",		"के तपाइ नेपाली बोल्नुहुन्छ?"},
 		{ "nl",		"Pa's wijze lynx bezag vroom het fikse aquaduct." },
+		{ "or",		"ଆପଣ ଓଡ଼ିଆ କୁହନ୍ତି କି? "},
+		{ "pa",		"ਖੁਸ਼ੀ ਤੁਹਾਨੂੰ ਮੀਟਿੰਗ ਲਈ"},
 		{ "pl",		"Pójdźże, kiń tę chmurność w głąb flaszy!" },
 		{ "pt",		"À noite, vovô Kowalsky vê o ímã cair no pé do pingüim queixoso e vovó põe açúcar no chá de tâmaras do jabuti feliz." },
 		{ "ru",		"В чащах юга жил бы цитрус? Да, но фальшивый экземпляр!" },
 		{ "sv",		"Gud hjälpe qvickt Zorns mö få aw byxor" },
+		{ "ta",		"மகிழ்ச்சி நீங்கள் சந்தித்த"},
+		{ "te",		"ఆనందం మీరు సమావేశం"},
 		{ "ua",		"Чуєш їх, доцю, га? Кумедна ж ти, прощайся без ґольфів!" },
 		{ "zh-tw",	"秋風滑過拔地紅樓角落，誤見釣人低聲吟詠離騷。" },
 		{ NULL, NULL } };
@@ -350,6 +368,12 @@ asb_font_get_pixbuf (FT_Face ft_face,
 	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
 	cairo_show_text (cr, text);
 	pixbuf = gdk_pixbuf_get_from_surface (surface, 0, 0, width, height);
+	if (pixbuf == NULL) {
+		g_set_error_literal (error,
+				     ASB_PLUGIN_ERROR,
+				     ASB_PLUGIN_ERROR_FAILED,
+				     "Could not get font pixbuf");
+	}
 
 	cairo_destroy (cr);
 	cairo_font_face_destroy (font_face);
@@ -381,13 +405,19 @@ asb_font_get_caption (AsbApp *app)
  * asb_font_add_screenshot:
  */
 static gboolean
-asb_font_add_screenshot (AsbApp *app, FT_Face ft_face, GError **error)
+asb_font_add_screenshot (AsbPlugin *plugin, AsbApp *app, FT_Face ft_face,
+			 const gchar *cache_id, GError **error)
 {
+	AsImage *im_tmp;
+	AsScreenshot *ss_tmp;
+	GPtrArray *screenshots;
 	const gchar *cache_dir;
-	const gchar *mirror_uri;
+	const gchar *temp_dir;
 	const gchar *tmp;
+	guint i;
 	_cleanup_free_ gchar *basename = NULL;
 	_cleanup_free_ gchar *cache_fn = NULL;
+	_cleanup_free_ gchar *output_fn = NULL;
 	_cleanup_free_ gchar *caption = NULL;
 	_cleanup_free_ gchar *url_tmp = NULL;
 	_cleanup_object_unref_ AsImage *im = NULL;
@@ -399,10 +429,9 @@ asb_font_add_screenshot (AsbApp *app, FT_Face ft_face, GError **error)
 		return TRUE;
 
 	/* is in the cache */
-	cache_dir = asb_package_get_config (asb_app_get_package (app), "CacheDir");
-	cache_fn = g_strdup_printf ("%s/%s.png",
-				    cache_dir,
-				    as_app_get_id_filename (AS_APP (app)));
+	cache_dir = asb_context_get_cache_dir (plugin->ctx);
+	cache_fn = g_strdup_printf ("%s/screenshots/%s.png",
+				    cache_dir, cache_id);
 	if (g_file_test (cache_fn, G_FILE_TEST_EXISTS)) {
 		pixbuf = gdk_pixbuf_new_from_file (cache_fn, error);
 		if (pixbuf == NULL)
@@ -422,8 +451,13 @@ asb_font_add_screenshot (AsbApp *app, FT_Face ft_face, GError **error)
 		return FALSE;
 	}
 
-	mirror_uri = asb_package_get_config (asb_app_get_package (app),
-					     "MirrorURI");
+	/* save to the cache for next time */
+	if (!g_file_test (cache_fn, G_FILE_TEST_EXISTS)) {
+		if (!gdk_pixbuf_save (pixbuf, cache_fn, "png", error, NULL))
+			return FALSE;
+	}
+
+	/* copy it to the screenshot directory */
 	im = as_image_new ();
 	as_image_set_pixbuf (im, pixbuf);
 	as_image_set_kind (im, AS_IMAGE_KIND_SOURCE);
@@ -431,12 +465,36 @@ asb_font_add_screenshot (AsbApp *app, FT_Face ft_face, GError **error)
 				    as_app_get_id_filename (AS_APP (app)),
 				    as_image_get_md5 (im));
 	as_image_set_basename (im, basename);
-	url_tmp = g_build_filename (mirror_uri,
-				    "source",
+	url_tmp = g_build_filename ("file://",
 				    basename,
 				    NULL);
 	as_image_set_url (im, url_tmp, -1);
 
+	/* put this in a special place so it gets packaged up */
+	temp_dir = asb_context_get_temp_dir (plugin->ctx);
+	output_fn = g_build_filename (temp_dir, "screenshots", basename, NULL);
+	if (!gdk_pixbuf_save (pixbuf, output_fn, "png", error, NULL))
+		return FALSE;
+
+	/* check the screenshot does not already exist */
+	screenshots = as_app_get_screenshots (AS_APP (app));
+	for (i = 0; i < screenshots->len; i++) {
+		ss_tmp = g_ptr_array_index (screenshots, i);
+		im_tmp = as_screenshot_get_source (ss_tmp);
+		if (im_tmp == NULL)
+			continue;
+		if (g_strcmp0 (as_image_get_md5 (im_tmp),
+			       as_image_get_md5 (im)) == 0) {
+			g_set_error (error,
+				     ASB_PLUGIN_ERROR,
+				     ASB_PLUGIN_ERROR_FAILED,
+				     "Font screenshot already exists with hash %s",
+				     as_image_get_md5 (im));
+			return FALSE;
+		}
+	}
+
+	/* add screenshot */
 	ss = as_screenshot_new ();
 	as_screenshot_set_kind (ss, AS_SCREENSHOT_KIND_DEFAULT);
 	as_screenshot_add_image (ss, im);
@@ -564,16 +622,22 @@ asb_plugin_font_app (AsbPlugin *plugin, AsbApp *app,
 	FT_Library library = NULL;
 	const gchar *tmp;
 	gboolean ret = TRUE;
+	guint i;
 	const FcPattern *pattern;
-	_cleanup_free_ gchar *app_id = NULL;
+	_cleanup_free_ gchar *cache_id = NULL;
 	_cleanup_free_ gchar *comment = NULL;
 	_cleanup_free_ gchar *icon_filename = NULL;
 	_cleanup_object_unref_ GdkPixbuf *pixbuf = NULL;
 
-	/* load font */
+	/* create a new fontconfig configuration */
 	config = FcConfigCreate ();
+
+	/* ensure that default configuration and fonts are not loaded */
+	FcConfigSetCurrent (config);
+
+	/* add just this one font */
 	ret = FcConfigAppFontAddFile (config, (FcChar8 *) filename);
-	if (!ret) {
+	if (FALSE && !ret) { /* FIXME: fails since f22 even for success */
 		g_set_error (error,
 			     ASB_PLUGIN_ERROR,
 			     ASB_PLUGIN_ERROR_FAILED,
@@ -602,8 +666,15 @@ asb_plugin_font_app (AsbPlugin *plugin, AsbApp *app,
 		goto out;
 	}
 
+	/* use the filename as the cache-id */
+	cache_id = g_path_get_basename (filename);
+	for (i = 0; cache_id[i] != '\0'; i++) {
+		if (g_ascii_isalnum (cache_id[i]))
+			continue;
+		cache_id[i] = '_';
+	}
+
 	/* create app that might get merged later */
-	app_id = g_path_get_basename (filename);
 	as_app_add_category (AS_APP (app), "Addons", -1);
 	as_app_add_category (AS_APP (app), "Fonts", -1);
 	if (as_app_get_name (AS_APP (app), NULL) == NULL)
@@ -617,7 +688,7 @@ asb_plugin_font_app (AsbPlugin *plugin, AsbApp *app,
 	asb_font_add_languages (app, pattern);
 	asb_font_add_metadata (app, ft_face);
 	asb_font_fix_metadata (app);
-	ret = asb_font_add_screenshot (app, ft_face, error);
+	ret = asb_font_add_screenshot (plugin, app, ft_face, cache_id, error);
 	if (!ret)
 		goto out;
 
@@ -643,7 +714,7 @@ asb_plugin_font_app (AsbPlugin *plugin, AsbApp *app,
 		}
 
 		/* add icon */
-		if (asb_context_get_hidpi_enabled (plugin->ctx)) {
+		if (asb_context_get_flag (plugin->ctx, ASB_CONTEXT_FLAG_HIDPI_ICONS)) {
 			icon_filename = g_strdup_printf ("64x64/%s.png",
 							 as_app_get_id_filename (AS_APP (app)));
 		} else {
