@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2014-2015 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -48,7 +48,7 @@ asb_plugin_add_globs (AsbPlugin *plugin, GPtrArray *globs)
 gboolean
 asb_plugin_check_filename (AsbPlugin *plugin, const gchar *filename)
 {
-	if (fnmatch ("/usr/lib64/gstreamer-1.0/libgst*.so", filename, 0) == 0)
+	if (asb_plugin_match_glob ("/usr/lib64/gstreamer-1.0/libgst*.so", filename))
 		return TRUE;
 	return FALSE;
 }
@@ -160,7 +160,7 @@ asb_plugin_process (AsbPlugin *plugin,
 	as_app_set_id_kind (AS_APP (app), AS_ID_KIND_CODEC);
 	as_app_set_name (AS_APP (app), "C", "GStreamer Multimedia Codecs", -1);
 	asb_app_set_requires_appdata (app, TRUE);
-	asb_app_set_hidpi_enabled (app, asb_context_get_hidpi_enabled (plugin->ctx));
+	asb_app_set_hidpi_enabled (app, asb_context_get_flag (plugin->ctx, ASB_CONTEXT_FLAG_HIDPI_ICONS));
 	as_app_add_category (AS_APP (app), "Addons", -1);
 	as_app_add_category (AS_APP (app), "Codecs", -1);
 
