@@ -87,56 +87,6 @@ asb_plugin_destroy (AsbPlugin *plugin)
 	g_ptr_array_unref (plugin->priv->project_groups);
 }
 
-/**
- * asb_plugin_hardcoded_sort_screenshots_cb:
- */
-static gint
-asb_plugin_hardcoded_sort_screenshots_cb (gconstpointer a, gconstpointer b)
-{
-	return g_strcmp0 ((gchar *) a, (gchar *) b);
-}
-
-/**
- * asb_plugin_hardcoded_add_screenshots:
- */
-static gboolean
-asb_plugin_hardcoded_add_screenshots (AsbApp *app,
-				      const gchar *location,
-				      GError **error)
-{
-	const gchar *tmp;
-	gboolean ret = TRUE;
-	GList *l;
-	GList *list = NULL;
-	_cleanup_dir_close_ GDir *dir = NULL;
-
-	/* scan for files */
-	dir = g_dir_open (location, 0, error);
-	if (dir == NULL) {
-		ret = FALSE;
-		goto out;
-	}
-	while ((tmp = g_dir_read_name (dir)) != NULL) {
-		if (!g_str_has_suffix (tmp, ".png"))
-			continue;
-		list = g_list_prepend (list, g_build_filename (location, tmp, NULL));
-	}
-	list = g_list_sort (list, asb_plugin_hardcoded_sort_screenshots_cb);
-	for (l = list; l != NULL; l = l->next) {
-		tmp = l->data;
-		asb_package_log (asb_app_get_package (app),
-				 ASB_PACKAGE_LOG_LEVEL_DEBUG,
-				 "Adding extra screenshot: %s", tmp);
-		ret = asb_app_add_screenshot_source (app, tmp, error);
-		if (!ret)
-			goto out;
-	}
-	as_app_add_metadata (AS_APP (app), "DistroScreenshots", NULL, -1);
-out:
-	g_list_free_full (list, g_free);
-	return ret;
-}
-
 /**
  * asb_plugin_process_app:
  */
@@ -149,7 +99,7 @@ asb_plugin_process_app (AsbPlugin *plugin,
 {
 	const gchar *tmp;
 	AsRelease *release;
-	gchar **deps;
+	GPtrArray *deps;
 	gchar **filelist;
 	GPtrArray *releases;
 	guint i;
@@ -203,8 +153,7 @@ asb_plugin_process_app (AsbPlugin *plugin,
 	/* look for any installed docs */
 	filelist = asb_package_get_filelist (pkg);
 	for (i = 0; filelist[i] != NULL; i++) {
-		if (g_str_has_prefix (filelist[i],
-				      "/usr/share/help/")) {
+		if (asb_plugin_match_glob ("/usr/share/help/*", filelist[i])) {
 			as_app_add_kudo_kind (AS_APP (app),
 					      AS_KUDO_KIND_USER_DOCS);
 			break;
@@ -213,8 +162,8 @@ asb_plugin_process_app (AsbPlugin *plugin,
 
 	/* look for a shell search provider */
 	for (i = 0; filelist[i] != NULL; i++) {
-		if (g_str_has_prefix (filelist[i],
-				      "/usr/share/gnome-shell/search-providers/")) {
+		if (asb_plugin_match_glob ("/usr/share/gnome-shell/search-providers/*",
+					   filelist[i])) {
 			as_app_add_kudo_kind (AS_APP (app),
 					      AS_KUDO_KIND_SEARCH_PROVIDER);
 			break;
@@ -223,8 +172,14 @@ asb_plugin_process_app (AsbPlugin *plugin,
 
 	/* look for a high contrast icon */
 	for (i = 0; filelist[i] != NULL; i++) {
-		if (g_str_has_prefix (filelist[i],
-				      "/usr/share/icons/HighContrast/")) {
+		if (asb_plugin_match_glob ("/usr/share/icons/HighContrast/*",
+					   filelist[i])) {
+			as_app_add_kudo_kind (AS_APP (app),
+					      AS_KUDO_KIND_HIGH_CONTRAST);
+			break;
+		}
+		if (asb_plugin_match_glob ("/usr/share/icons/hicolor/symbolic/apps/*.svg",
+					   filelist[i])) {
 			as_app_add_kudo_kind (AS_APP (app),
 					      AS_KUDO_KIND_HIGH_CONTRAST);
 			break;
@@ -233,9 +188,10 @@ asb_plugin_process_app (AsbPlugin *plugin,
 
 	/* look for a modern toolkit */
 	deps = asb_package_get_deps (pkg);
-	for (i = 0; deps != NULL && deps[i] != NULL; i++) {
-		if (g_strcmp0 (deps[i], "libgtk-3.so.0") == 0 ||
-		    g_strcmp0 (deps[i], "libQt5Core.so.5") == 0) {
+	for (i = 0; i < deps->len; i++) {
+		tmp = g_ptr_array_index (deps, i);
+		if (g_strcmp0 (tmp, "libgtk-3.so.0") == 0 ||
+		    g_strcmp0 (tmp, "libQt5Core.so.5") == 0) {
 			as_app_add_kudo_kind (AS_APP (app),
 					      AS_KUDO_KIND_MODERN_TOOLKIT);
 			break;
@@ -243,36 +199,37 @@ asb_plugin_process_app (AsbPlugin *plugin,
 	}
 
 	/* look for ancient toolkits */
-	for (i = 0; deps != NULL && deps[i] != NULL; i++) {
-		if (g_strcmp0 (deps[i], "libgtk-1.2.so.0") == 0) {
+	for (i = 0; i < deps->len; i++) {
+		tmp = g_ptr_array_index (deps, i);
+		if (g_strcmp0 (tmp, "libgtk-1.2.so.0") == 0) {
 			as_app_add_veto (AS_APP (app), "Uses obsolete GTK1 toolkit");
 			break;
 		}
-		if (g_strcmp0 (deps[i], "libglib-1.2.so.0") == 0) {
+		if (g_strcmp0 (tmp, "libglib-1.2.so.0") == 0) {
 			as_app_add_veto (AS_APP (app), "Uses obsolete GLib library");
 			break;
 		}
-		if (g_strcmp0 (deps[i], "libqt-mt.so.3") == 0) {
+		if (g_strcmp0 (tmp, "libqt-mt.so.3") == 0) {
 			as_app_add_veto (AS_APP (app), "Uses obsolete QT3 toolkit");
 			break;
 		}
-		if (g_strcmp0 (deps[i], "liblcms.so.1") == 0) {
+		if (g_strcmp0 (tmp, "liblcms.so.1") == 0) {
 			as_app_add_veto (AS_APP (app), "Uses obsolete LCMS library");
 			break;
 		}
-		if (g_strcmp0 (deps[i], "libelektra.so.4") == 0) {
+		if (g_strcmp0 (tmp, "libelektra.so.4") == 0) {
 			as_app_add_veto (AS_APP (app), "Uses obsolete Elektra library");
 			break;
 		}
-		if (g_strcmp0 (deps[i], "libXt.so.6") == 0) {
+		if (g_strcmp0 (tmp, "libXt.so.6") == 0) {
 			asb_app_add_requires_appdata (app, "Uses obsolete X11 toolkit");
 			break;
 		}
-		if (g_strcmp0 (deps[i], "Xvfb") == 0) {
+		if (g_strcmp0 (tmp, "Xvfb") == 0) {
 			asb_app_add_requires_appdata (app, "Uses obsolete Xvfb");
 			break;
 		}
-		if (g_strcmp0 (deps[i], "wine-core") == 0) {
+		if (g_strcmp0 (tmp, "wine-core") == 0) {
 			asb_app_add_requires_appdata (app, "Uses wine");
 			break;
 		}
@@ -308,17 +265,6 @@ asb_plugin_process_app (AsbPlugin *plugin,
 		}
 	}
 
-	/* do any extra screenshots exist */
-	tmp = asb_package_get_config (pkg, "ScreenshotsExtra");
-	if (tmp != NULL) {
-		_cleanup_free_ gchar *dirname = NULL;
-		dirname = g_build_filename (tmp, as_app_get_id_filename (AS_APP (app)), NULL);
-		if (g_file_test (dirname, G_FILE_TEST_EXISTS)) {
-			if (!asb_plugin_hardcoded_add_screenshots (app, dirname, error))
-				return FALSE;
-		}
-	}
-
 	/* a ConsoleOnly category means we require AppData */
 	if (as_app_has_category (AS_APP(app), "ConsoleOnly"))
 		asb_app_add_requires_appdata (app, "ConsoleOnly");
