@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2014-2015 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -28,6 +28,7 @@
 
 #include <glib-object.h>
 
+#include "as-bundle.h"
 #include "as-enums.h"
 #include "as-icon.h"
 #include "as-provide.h"
@@ -162,6 +163,7 @@ typedef enum {
  * @AS_APP_SOURCE_KIND_DESKTOP:			Sourced from a desktop file
  * @AS_APP_SOURCE_KIND_APPDATA:			Sourced from a AppData file
  * @AS_APP_SOURCE_KIND_METAINFO:		Sourced from a MetaInfo file
+ * @AS_APP_SOURCE_KIND_INF:			Sourced from a inf file
  *
  * The source kind.
  **/
@@ -171,6 +173,7 @@ typedef enum {
 	AS_APP_SOURCE_KIND_DESKTOP,			/* Since: 0.1.4 */
 	AS_APP_SOURCE_KIND_APPDATA,			/* Since: 0.1.4 */
 	AS_APP_SOURCE_KIND_METAINFO,			/* Since: 0.1.7 */
+	AS_APP_SOURCE_KIND_INF,				/* Since: 0.3.5 */
 	/*< private >*/
 	AS_APP_SOURCE_KIND_LAST
 } AsAppSourceKind;
@@ -226,6 +229,7 @@ GPtrArray	*as_app_get_extends		(AsApp		*app);
 GPtrArray	*as_app_get_keywords		(AsApp		*app,
 						 const gchar	*locale);
 GPtrArray	*as_app_get_kudos		(AsApp		*app);
+GPtrArray	*as_app_get_permissions		(AsApp		*app);
 GPtrArray	*as_app_get_mimetypes		(AsApp		*app);
 GPtrArray	*as_app_get_pkgnames		(AsApp		*app);
 GPtrArray	*as_app_get_architectures	(AsApp		*app);
@@ -233,6 +237,7 @@ GPtrArray	*as_app_get_releases		(AsApp		*app);
 GPtrArray	*as_app_get_provides		(AsApp		*app);
 GPtrArray	*as_app_get_screenshots		(AsApp		*app);
 GPtrArray	*as_app_get_icons		(AsApp		*app);
+GPtrArray	*as_app_get_bundles		(AsApp		*app);
 GHashTable	*as_app_get_names		(AsApp		*app);
 GHashTable	*as_app_get_comments		(AsApp		*app);
 GHashTable	*as_app_get_developer_names	(AsApp		*app);
@@ -272,6 +277,8 @@ gboolean	 as_app_has_kudo		(AsApp		*app,
 						 const gchar	*kudo);
 gboolean	 as_app_has_kudo_kind		(AsApp		*app,
 						 AsKudoKind	 kudo);
+gboolean	 as_app_has_permission		(AsApp		*app,
+						 const gchar	*permission);
 
 /* setters */
 void		 as_app_set_id			(AsApp		*app,
@@ -337,6 +344,9 @@ void		 as_app_add_kudo		(AsApp		*app,
 						 gssize		 kudo_len);
 void		 as_app_add_kudo_kind		(AsApp		*app,
 						 AsKudoKind	 kudo_kind);
+void		 as_app_add_permission		(AsApp		*app,
+						 const gchar	*permission,
+						 gssize		 permission_len);
 void		 as_app_add_mimetype		(AsApp		*app,
 						 const gchar	*mimetype,
 						 gssize		 mimetype_len);
@@ -354,6 +364,8 @@ void		 as_app_add_screenshot		(AsApp		*app,
 						 AsScreenshot	*screenshot);
 void		 as_app_add_icon		(AsApp		*app,
 						 AsIcon		*icon);
+void		 as_app_add_bundle		(AsApp		*app,
+						 AsBundle	*bundle);
 void		 as_app_add_language		(AsApp		*app,
 						 gint		 percentage,
 						 const gchar	*locale,
@@ -406,6 +418,10 @@ AsIcon		*as_app_get_icon_default	(AsApp		*app);
 AsIcon		*as_app_get_icon_for_size	(AsApp		*app,
 						 guint		 width,
 						 guint		 height);
+AsBundle	*as_app_get_bundle_default	(AsApp		*app);
+AsRelease	*as_app_get_release		(AsApp		*app,
+						 const gchar	*version);
+AsRelease	*as_app_get_release_default	(AsApp		*app);
 gboolean	 as_app_convert_icons		(AsApp		*app,
 						 AsIconKind	 kind,
 						 GError		**error);
