@@ -511,7 +511,11 @@ as_node_to_xml (const GNode *node, AsNodeToXmlFlags flags)
 {
 	GString *xml;
 	const GNode *l;
-	guint depth_offset = g_node_depth ((GNode *) node) + 1;
+	guint depth_offset;
+
+	g_return_val_if_fail (node != NULL, NULL);
+
+	depth_offset = g_node_depth ((GNode *) node) + 1;
 	xml = g_string_new ("");
 	if ((flags & AS_NODE_TO_XML_FLAG_ADD_HEADER) > 0)
 		g_string_append (xml, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
@@ -1870,7 +1874,19 @@ as_node_get_localized_unwrap (const GNode *node, GError **error)
 	_cleanup_hashtable_unref_ GHashTable *hash = NULL;
 	_cleanup_list_free_ GList *keys = NULL;
 
+	g_return_val_if_fail (node != NULL, NULL);
+
 	/* work out what kind of normalization this is */
+	xml_lang = as_node_get_attribute (node, "xml:lang");
+	if (xml_lang != NULL && node->children != NULL) {
+		str = as_node_to_xml (node->children, AS_NODE_TO_XML_FLAG_NONE);
+		results = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+		g_hash_table_insert (results,
+				     g_strdup (xml_lang),
+				     g_strdup (str->str));
+		g_string_free (str, TRUE);
+		return results;
+	}
 	for (tmp = node->children; tmp != NULL; tmp = tmp->next) {
 		data = tmp->data;
 		if (g_strcmp0 (data->name, "ul") == 0 ||
@@ -1905,3 +1921,123 @@ as_node_get_localized_unwrap (const GNode *node, GError **error)
 	}
 	return results;
 }
+
+/* helper struct */
+struct _AsNodeContext {
+	AsAppSourceKind	 source_kind;
+	AsAppSourceKind	 output;
+	gdouble		 version;
+};
+
+/**
+ * as_node_context_new: (skip)
+ *
+ * Creates a new node context with default values.
+ *
+ * Returns: a #AsNodeContext
+ *
+ * Since: 0.3.6
+ **/
+AsNodeContext *
+as_node_context_new (void)
+{
+	AsNodeContext *ctx;
+	ctx = g_new0 (AsNodeContext, 1);
+	ctx->version = 0.f;
+	ctx->source_kind = AS_APP_SOURCE_KIND_APPSTREAM;
+	ctx->output = AS_APP_SOURCE_KIND_UNKNOWN;
+	return ctx;
+}
+
+/**
+ * as_node_context_get_version: (skip)
+ * @ctx: a #AsNodeContext.
+ *
+ * Gets the AppStream API version used when parsing or inserting nodes.
+ *
+ * Returns: version number
+ *
+ * Since: 0.3.6
+ **/
+gdouble
+as_node_context_get_version (AsNodeContext *ctx)
+{
+	return ctx->version;
+}
+
+/**
+ * as_node_context_set_version: (skip)
+ * @ctx: a #AsNodeContext.
+ * @version: an API version number to target.
+ *
+ * Sets the AppStream API version used when parsing or inserting nodes.
+ *
+ * Since: 0.3.6
+ **/
+void
+as_node_context_set_version (AsNodeContext *ctx, gdouble version)
+{
+	ctx->version = version;
+}
+
+/**
+ * as_node_context_get_source_kind: (skip)
+ * @ctx: a #AsNodeContext.
+ *
+ * Gets the AppStream API source_kind used when parsing nodes.
+ *
+ * Returns: source_kind number
+ *
+ * Since: 0.3.6
+ **/
+AsAppSourceKind
+as_node_context_get_source_kind (AsNodeContext *ctx)
+{
+	return ctx->source_kind;
+}
+
+/**
+ * as_node_context_set_source_kind: (skip)
+ * @ctx: a #AsNodeContext.
+ * @source_kind: an API source_kind number to target.
+ *
+ * Sets the AppStream API source_kind used when parsing nodes.
+ *
+ * Since: 0.3.6
+ **/
+void
+as_node_context_set_source_kind (AsNodeContext *ctx, AsAppSourceKind source_kind)
+{
+	ctx->source_kind = source_kind;
+}
+
+/**
+ * as_node_context_get_output: (skip)
+ * @ctx: a #AsNodeContext.
+ *
+ * Gets the AppStream API destination kind used when inserting nodes.
+ *
+ * Returns: output format, e.g. %AS_APP_SOURCE_KIND_APPDATA
+ *
+ * Since: 0.3.6
+ **/
+AsAppSourceKind
+as_node_context_get_output (AsNodeContext *ctx)
+{
+	return ctx->output;
+}
+
+/**
+ * as_node_context_set_output: (skip)
+ * @ctx: a #AsNodeContext.
+ * @output: an output kind, e.g. %AS_APP_SOURCE_KIND_APPDATA
+ *
+ * Sets the AppStream API destination kind used when inserting nodes.
+ *
+ * Since: 0.3.6
+ **/
+void
+as_node_context_set_output (AsNodeContext *ctx, AsAppSourceKind output)
+{
+	ctx->output = output;
+}
