@@ -66,6 +66,7 @@ struct _AsStorePrivate
 	GHashTable		*metadata_indexes;	/* GHashTable{key} */
 	AsStoreAddFlags		 add_flags;
 	AsStoreProblems		 problems;
+	guint32			 filter;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsStore, as_store, G_TYPE_OBJECT)
@@ -167,6 +168,47 @@ as_store_class_init (AsStoreClass *klass)
 	object_class->finalize = as_store_finalize;
 }
 
+/**
+ * as_store_add_filter:
+ * @store: a #AsStore instance.
+ * @kind: a #AsIdKind, e.g. %AS_ID_KIND_FIRMWARE
+ *
+ * Adds a filter to the store so that only components of this type are
+ * loaded into the store. This may be useful if the client is only interested
+ * in certain types of component, or not interested in loading components
+ * it cannot process.
+ *
+ * If no filter is set then all types of components are loaded.
+ *
+ * Since: 0.3.5
+ **/
+void
+as_store_add_filter (AsStore *store, AsIdKind kind)
+{
+	AsStorePrivate *priv = GET_PRIVATE (store);
+	priv->filter |= 1 << kind;
+}
+
+/**
+ * as_store_remove_filter:
+ * @store: a #AsStore instance.
+ * @kind: a #AsIdKind, e.g. %AS_ID_KIND_FIRMWARE
+ *
+ * Removed a filter from the store so that components of this type are no longer
+ * loaded into the store. This may be useful if the client is only interested
+ * in certain types of component.
+ *
+ * If all filters are removed then all types of components are loaded.
+ *
+ * Since: 0.3.5
+ **/
+void
+as_store_remove_filter (AsStore *store, AsIdKind kind)
+{
+	AsStorePrivate *priv = GET_PRIVATE (store);
+	priv->filter &= ~(1 << kind);
+}
+
 /**
  * as_store_get_size:
  * @store: a #AsStore instance.
@@ -583,6 +625,7 @@ as_store_from_root (AsStore *store,
 	GNode *apps;
 	GNode *n;
 	const gchar *tmp;
+	_cleanup_free_ AsNodeContext *ctx = NULL;
 	_cleanup_free_ gchar *icon_path = NULL;
 
 	g_return_val_if_fail (AS_IS_STORE (store), FALSE);
@@ -623,16 +666,29 @@ as_store_from_root (AsStore *store,
 					      priv->origin,
 					      NULL);
 	}
+	ctx = as_node_context_new ();
 	for (n = apps->children; n != NULL; n = n->next) {
 		_cleanup_error_free_ GError *error_local = NULL;
 		_cleanup_object_unref_ AsApp *app = NULL;
 		if (as_node_get_tag (n) != AS_TAG_APPLICATION)
 			continue;
+
+		/* do the filtering here */
+		if (priv->filter != 0) {
+			if (g_strcmp0 (as_node_get_name (n), "component") == 0) {
+				AsIdKind kind_tmp;
+				tmp = as_node_get_attribute (n, "type");
+				kind_tmp = as_id_kind_from_string (tmp);
+				if ((priv->filter & (1 << kind_tmp)) == 0)
+					continue;
+			}
+		}
+
 		app = as_app_new ();
 		if (icon_path != NULL)
 			as_app_set_icon_path (app, icon_path, -1);
 		as_app_set_source_kind (app, AS_APP_SOURCE_KIND_APPSTREAM);
-		if (!as_app_node_parse (app, n, &error_local)) {
+		if (!as_app_node_parse (app, n, ctx, &error_local)) {
 			g_set_error (error,
 				     AS_STORE_ERROR,
 				     AS_STORE_ERROR_FAILED,
@@ -664,6 +720,7 @@ as_store_load_yaml_file (AsStore *store,
 	GNode *app_n;
 	GNode *n;
 	const gchar *tmp;
+	_cleanup_free_ AsNodeContext *ctx = NULL;
 	_cleanup_free_ gchar *icon_path = NULL;
 	_cleanup_yaml_unref_ GNode *root = NULL;
 
@@ -696,6 +753,7 @@ as_store_load_yaml_file (AsStore *store,
 	}
 
 	/* parse applications */
+	ctx = as_node_context_new ();
 	for (app_n = root->children->next; app_n != NULL; app_n = app_n->next) {
 		_cleanup_object_unref_ AsApp *app = NULL;
 		if (app_n->children == NULL)
@@ -704,7 +762,7 @@ as_store_load_yaml_file (AsStore *store,
 		if (icon_path != NULL)
 			as_app_set_icon_path (app, icon_path, -1);
 		as_app_set_source_kind (app, AS_APP_SOURCE_KIND_APPSTREAM);
-		if (!as_app_node_parse_dep11 (app, app_n, error))
+		if (!as_app_node_parse_dep11 (app, app_n, ctx, error))
 			return FALSE;
 		as_app_set_origin (app, priv->origin);
 		if (as_app_get_id (app) != NULL)
@@ -842,6 +900,7 @@ as_store_to_xml (AsStore *store, AsNodeToXmlFlags flags)
 	GString *xml;
 	guint i;
 	gchar version[6];
+	_cleanup_free_ AsNodeContext *ctx = NULL;
 
 	/* get XML text */
 	node_root = as_node_new ();
@@ -870,9 +929,12 @@ as_store_to_xml (AsStore *store, AsNodeToXmlFlags flags)
 	g_ptr_array_sort (priv->array, as_store_apps_sort_cb);
 
 	/* add applications */
+	ctx = as_node_context_new ();
+	as_node_context_set_version (ctx, priv->api_version);
+	as_node_context_set_output (ctx, AS_APP_SOURCE_KIND_APPSTREAM);
 	for (i = 0; i < priv->array->len; i++) {
 		app = g_ptr_array_index (priv->array, i);
-		as_app_node_insert (app, node_apps, priv->api_version);
+		as_app_node_insert (app, node_apps, ctx);
 	}
 	xml = as_node_to_xml (node_root, flags);
 	as_node_unref (node_root);
