@@ -37,6 +37,7 @@
 #include <libsoup/soup.h>
 #include <archive_entry.h>
 #include <archive.h>
+#include <stdlib.h>
 
 #include "as-app.h"
 #include "as-cleanup.h"
@@ -83,21 +84,132 @@ as_strndup (const gchar *text, gssize text_len)
 }
 
 /**
- * as_markup_convert_simple:
+ * as_markup_strsplit_words:
+ * @text: the text to split.
+ * @line_len: the maximum length of the output line
+ *
+ * Splits up a long line into an array of smaller strings, each being no longer
+ * than @line_len. Words are not split.
+ *
+ * Returns: (transfer full): lines, or %NULL in event of an error
+ *
+ * Since: 0.3.5
+ **/
+gchar **
+as_markup_strsplit_words (const gchar *text, guint line_len)
+{
+	GPtrArray *lines;
+	guint i;
+	_cleanup_string_free_ GString *curline = NULL;
+	_cleanup_strv_free_ gchar **tokens = NULL;
+
+	/* sanity check */
+	if (text == NULL || text[0] == '\0')
+		return NULL;
+	if (line_len == 0)
+		return NULL;
+
+	lines = g_ptr_array_new ();
+	curline = g_string_new ("");
+
+	/* tokenize the string */
+	tokens = g_strsplit (text, " ", -1);
+	for (i = 0; tokens[i] != NULL; i++) {
+
+		/* current line plus new token is okay */
+		if (curline->len + strlen (tokens[i]) < line_len) {
+			g_string_append_printf (curline, "%s ", tokens[i]);
+			continue;
+		}
+
+		/* too long, so remove space, add newline and dump */
+		if (curline->len > 0)
+			g_string_truncate (curline, curline->len - 1);
+		g_string_append (curline, "\n");
+		g_ptr_array_add (lines, g_strdup (curline->str));
+		g_string_truncate (curline, 0);
+		g_string_append_printf (curline, "%s ", tokens[i]);
+
+	}
+
+	/* any incomplete line? */
+	if (curline->len > 0) {
+		g_string_truncate (curline, curline->len - 1);
+		g_string_append (curline, "\n");
+		g_ptr_array_add (lines, g_strdup (curline->str));
+	}
+
+	g_ptr_array_add (lines, NULL);
+	return (gchar **) g_ptr_array_free (lines, FALSE);
+}
+
+/**
+ * as_markup_render_para:
+ **/
+static void
+as_markup_render_para (GString *str, AsMarkupConvertFormat format, const gchar *data)
+{
+	guint i;
+	_cleanup_strv_free_ gchar **spl = NULL;
+
+	if (str->len > 0)
+		g_string_append (str, "\n");
+	switch (format) {
+	case AS_MARKUP_CONVERT_FORMAT_SIMPLE:
+		g_string_append_printf (str, "%s\n", data);
+		break;
+	case AS_MARKUP_CONVERT_FORMAT_MARKDOWN:
+		/* break to 80 chars */
+		spl = as_markup_strsplit_words (data, 80);
+		for (i = 0; spl[i] != NULL; i++)
+			g_string_append (str, spl[i]);
+		break;
+	default:
+		break;
+	}
+}
+
+/**
+ * as_markup_render_li:
+ **/
+static void
+as_markup_render_li (GString *str, AsMarkupConvertFormat format, const gchar *data)
+{
+	guint i;
+	_cleanup_strv_free_ gchar **spl = NULL;
+
+	switch (format) {
+	case AS_MARKUP_CONVERT_FORMAT_SIMPLE:
+		g_string_append_printf (str, " • %s\n", data);
+		break;
+	case AS_MARKUP_CONVERT_FORMAT_MARKDOWN:
+		/* break to 80 chars, leaving room for the dot/indent */
+		spl = as_markup_strsplit_words (data, 80 - 3);
+		g_string_append_printf (str, " * %s", spl[0]);
+		for (i = 1; spl[i] != NULL; i++)
+			g_string_append_printf (str, "   %s", spl[i]);
+		break;
+	default:
+		break;
+	}
+}
+
+/**
+ * as_markup_convert:
  * @markup: the text to copy.
  * @markup_len: the length of @markup, or -1 if @markup is NULL terminated.
+ * @format: the #AsMarkupConvertFormat, e.g. %AS_MARKUP_CONVERT_FORMAT_MARKDOWN
  * @error: A #GError or %NULL
  *
  * Converts an XML description into a printable form.
  *
  * Returns: (transfer full): a newly allocated %NULL terminated string
  *
- * Since: 0.1.0
+ * Since: 0.3.5
  **/
 gchar *
-as_markup_convert_simple (const gchar *markup,
-			  gssize markup_len,
-			  GError **error)
+as_markup_convert (const gchar *markup, gssize markup_len,
+		   AsMarkupConvertFormat format, GError **error)
 {
 	GNode *tmp;
 	GNode *tmp_c;
@@ -124,9 +236,7 @@ as_markup_convert_simple (const gchar *markup,
 
 		tag = as_node_get_name (tmp);
 		if (g_strcmp0 (tag, "p") == 0) {
-			if (str->len > 0)
-				g_string_append (str, "\n");
-			g_string_append_printf (str, "%s\n", as_node_get_data (tmp));
+			as_markup_render_para (str, format, as_node_get_data (tmp));
 
 		/* loop on the children */
 		} else if (g_strcmp0 (tag, "ul") == 0 ||
@@ -134,9 +244,8 @@ as_markup_convert_simple (const gchar *markup,
 			for (tmp_c = tmp->children; tmp_c != NULL; tmp_c = tmp_c->next) {
 				tag_c = as_node_get_name (tmp_c);
 				if (g_strcmp0 (tag_c, "li") == 0) {
-					g_string_append_printf (str,
-								" • %s\n",
-								as_node_get_data (tmp_c));
+					as_markup_render_li (str, format,
+							     as_node_get_data (tmp_c));
 				} else {
 					/* only <li> is valid in lists */
 					g_set_error (error,
@@ -163,6 +272,28 @@ as_markup_convert_simple (const gchar *markup,
 	return g_strdup (str->str);
 }
 
+/**
+ * as_markup_convert_simple:
+ * @markup: the text to copy.
+ * @markup_len: the length of @markup, or -1 if @markup is NULL terminated.
+ * @error: A #GError or %NULL
+ *
+ * Converts an XML description into a printable form.
+ *
+ * Returns: (transfer full): a newly allocated %NULL terminated string
+ *
+ * Since: 0.1.0
+ **/
+gchar *
+as_markup_convert_simple (const gchar *markup,
+			  gssize markup_len,
+			  GError **error)
+{
+	return as_markup_convert (markup, markup_len,
+				  AS_MARKUP_CONVERT_FORMAT_SIMPLE,
+				  error);
+}
+
 /**
  * as_hash_lookup_by_locale:
  * @hash: a #GHashTable.
@@ -875,6 +1006,7 @@ as_utils_find_icon_filename_full (const gchar *destdir,
 				 "status",
 				 "stock",
 				 NULL };
+	_cleanup_free_ gchar *prefix = NULL;
 
 	/* fallback */
 	if (destdir == NULL)
@@ -895,6 +1027,20 @@ as_utils_find_icon_filename_full (const gchar *destdir,
 		return g_strdup (tmp);
 	}
 
+	/* all now found in the prefix */
+	prefix = g_strdup_printf ("%s/usr", destdir);
+	if (!g_file_test (prefix, G_FILE_TEST_EXISTS)) {
+		g_free (prefix);
+		prefix = g_strdup_printf ("%s/files", destdir);
+	}
+	if (!g_file_test (prefix, G_FILE_TEST_EXISTS)) {
+		g_set_error (error,
+			     AS_UTILS_ERROR,
+			     AS_UTILS_ERROR_FAILED,
+			     "Failed to find icon in prefix %s", search);
+		return NULL;
+	}
+
 	/* icon theme apps */
 	sizes = flags & AS_UTILS_FIND_ICON_HI_DPI ? sizes_hi_dpi : sizes_lo_dpi;
 	for (k = 0; theme_dirs[k] != NULL; k++) {
@@ -902,9 +1048,9 @@ as_utils_find_icon_filename_full (const gchar *destdir,
 			for (m = 0; types[m] != NULL; m++) {
 				for (j = 0; supported_ext[j] != NULL; j++) {
 					_cleanup_free_ gchar *tmp = NULL;
-					tmp = g_strdup_printf ("%s/usr/share/icons/"
+					tmp = g_strdup_printf ("%s/share/icons/"
 							       "%s/%s/%s/%s%s",
-							       destdir,
+							       prefix,
 							       theme_dirs[k],
 							       sizes[i],
 							       types[m],
@@ -921,8 +1067,8 @@ as_utils_find_icon_filename_full (const gchar *destdir,
 	for (i = 0; pixmap_dirs[i] != NULL; i++) {
 		for (j = 0; supported_ext[j] != NULL; j++) {
 			_cleanup_free_ gchar *tmp = NULL;
-			tmp = g_strdup_printf ("%s/usr/share/%s/%s%s",
-					       destdir,
+			tmp = g_strdup_printf ("%s/share/%s/%s%s",
+					       prefix,
 					       pixmap_dirs[i],
 					       search,
 					       supported_ext[j]);
@@ -1378,3 +1524,67 @@ as_utils_search_tokenize (const gchar *search)
 	}
 	return values;
 }
+
+/**
+ * as_utils_vercmp:
+ * @version_a: the release version, e.g. 1.2.3
+ * @version_b: the release version, e.g. 1.2.3.1
+ *
+ * Compares version numbers for sorting. This function cannot deal with version
+ * strings that do not contain numbers, for instance "rev2706" or "1.2_alpha".
+ *
+ * Returns: -1 if a < b, +1 if a > b, 0 if they are equal, and %G_MAXINT on error
+ *
+ * Since: 0.3.5
+ */
+gint
+as_utils_vercmp (const gchar *version_a, const gchar *version_b)
+{
+	gchar *endptr;
+	gint64 ver_a;
+	gint64 ver_b;
+	guint i;
+	guint longest_split;
+	_cleanup_strv_free_ gchar **split_a = NULL;
+	_cleanup_strv_free_ gchar **split_b = NULL;
+
+	/* sanity check */
+	if (version_a == NULL || version_b == NULL)
+		return G_MAXINT;
+
+	/* optimisation */
+	if (g_strcmp0 (version_a, version_b) == 0)
+		return 0;
+
+	/* split into sections, and try to parse */
+	split_a = g_strsplit (version_a, ".", -1);
+	split_b = g_strsplit (version_b, ".", -1);
+	longest_split = MAX (g_strv_length (split_a), g_strv_length (split_b));
+	for (i = 0; i < longest_split; i++) {
+
+		/* we lost or gained a dot */
+		if (split_a[i] == NULL)
+			return -1;
+		if (split_b[i] == NULL)
+			return 1;
+
+		/* compare integers */
+		ver_a = g_ascii_strtoll (split_a[i], &endptr, 10);
+		if (endptr != NULL && endptr[0] != '\0')
+			return G_MAXINT;
+		if (ver_a < 0)
+			return G_MAXINT;
+		ver_b = g_ascii_strtoll (split_b[i], &endptr, 10);
+		if (endptr != NULL && endptr[0] != '\0')
+			return G_MAXINT;
+		if (ver_b < 0)
+			return G_MAXINT;
+		if (ver_a < ver_b)
+			return -1;
+		if (ver_a > ver_b)
+			return 1;
+	}
+
+	/* we really shouldn't get here */
+	return 0;
+}
